@@ -5,7 +5,7 @@ Connection module for Amazon ELB
 .. versionadded:: 2014.7.0
 
 :configuration: This module accepts explicit elb credentials but can also utilize
-    IAM roles assigned to the instance trough Instance Profiles. Dynamic
+    IAM roles assigned to the instance through Instance Profiles. Dynamic
     credentials are then automatically obtained from AWS API and no further
     configuration is necessary. More Information available at:
 
@@ -57,6 +57,7 @@ log = logging.getLogger(__name__)
 # Import third party libs
 try:
     import boto
+    import boto.ec2  # pylint: enable=unused-import
     # connection settings were added in 2.33.0
     required_boto_version = '2.33.0'
     if (_LooseVersion(boto.__version__) <
@@ -64,7 +65,6 @@ try:
         msg = 'boto_elb requires boto {0}.'.format(required_boto_version)
         logging.debug(msg)
         raise ImportError()
-    import boto.ec2
     from boto.ec2.elb import HealthCheck
     from boto.ec2.elb.attributes import AccessLogAttribute
     from boto.ec2.elb.attributes import ConnectionDrainingAttribute
@@ -85,7 +85,7 @@ def __virtual__():
     Only load if boto libraries exist.
     '''
     if not HAS_BOTO:
-        return False
+        return (False, "The boto_elb module cannot be loaded: boto library not found")
     __utils__['boto.assign_funcs'](__name__, 'elb', module='ec2.elb', pack=__salt__)
     return True
 
@@ -94,7 +94,9 @@ def exists(name, region=None, key=None, keyid=None, profile=None):
     '''
     Check to see if an ELB exists.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.exists myelb region=us-east-1
     '''
@@ -109,15 +111,51 @@ def exists(name, region=None, key=None, keyid=None, profile=None):
             log.debug(msg)
             return False
     except boto.exception.BotoServerError as error:
-        log.debug(error)
+        log.warning(error)
         return False
 
 
+def get_all_elbs(region=None, key=None, keyid=None, profile=None):
+    '''
+    Return all load balancers associated with an account
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.get_all_elbs region=us-east-1
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    try:
+        return [e for e in conn.get_all_load_balancers()]
+    except boto.exception.BotoServerError as error:
+        log.warning(error)
+        return []
+
+
+def list_elbs(region=None, key=None, keyid=None, profile=None):
+    '''
+    Return names of all load balancers associated with an account
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.list_elbs region=us-east-1
+    '''
+
+    return [e.name for e in get_all_elbs(region=region, key=key, keyid=keyid,
+                                         profile=profile)]
+
+
 def get_elb_config(name, region=None, key=None, keyid=None, profile=None):
     '''
-    Check to see if an ELB exists.
+    Get an ELB configuration.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.exists myelb region=us-east-1
     '''
@@ -130,62 +168,96 @@ def get_elb_config(name, region=None, key=None, keyid=None, profile=None):
         ret['availability_zones'] = lb.availability_zones
         listeners = []
         for _listener in lb.listeners:
-            # Making this a list makes our life easier and is also the only way
-            # to include the certificate.
-            complex_listener = list(_listener.get_complex_tuple())
-            # boto, you're *killing* me with this. boto doesn't include the
-            # certificate when calling get_complex_tuple, so you need to also
-            # get the certificate. So. Much. Hate.
+            listener_dict = {}
+            listener_dict['elb_port'] = _listener.load_balancer_port
+            listener_dict['elb_protocol'] = _listener.protocol
+            listener_dict['instance_port'] = _listener.instance_port
+            listener_dict['instance_protocol'] = _listener.instance_protocol
+            listener_dict['policies'] = _listener.policy_names
             if _listener.ssl_certificate_id:
-                complex_listener.append(_listener.ssl_certificate_id)
-            listeners.append(complex_listener)
+                listener_dict['certificate'] = _listener.ssl_certificate_id
+            listeners.append(listener_dict)
         ret['listeners'] = listeners
+        backends = []
+        for _backend in lb.backends:
+            bs_dict = {}
+            bs_dict['instance_port'] = _backend.instance_port
+            bs_dict['policies'] = [p.policy_name for p in _backend.policies]
+            backends.append(bs_dict)
+        ret['backends'] = backends
         ret['subnets'] = lb.subnets
         ret['security_groups'] = lb.security_groups
         ret['scheme'] = lb.scheme
         ret['dns_name'] = lb.dns_name
+        ret['tags'] = _get_all_tags(conn, name)
+        lb_policy_lists = [
+            lb.policies.app_cookie_stickiness_policies,
+            lb.policies.lb_cookie_stickiness_policies,
+            lb.policies.other_policies
+            ]
+        policies = []
+        for policy_list in lb_policy_lists:
+            policies += [p.policy_name for p in policy_list]
+        ret['policies'] = policies
         return ret
     except boto.exception.BotoServerError as error:
         log.debug(error)
         return {}
 
 
-def create(name, availability_zones, listeners=None, subnets=None,
+def listener_dict_to_tuple(listener):
+    '''
+    Convert an ELB listener dict into a listener tuple used by certain parts of
+    the AWS ELB API.
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.listener_dict_to_tuple '{"elb_port":80,"instance_port":80,"elb_protocol":"HTTP"}'
+    '''
+    # We define all listeners as complex listeners.
+    if 'instance_protocol' not in listener:
+        instance_protocol = listener['elb_protocol'].upper()
+    else:
+        instance_protocol = listener['instance_protocol'].upper()
+    listener_tuple = [listener['elb_port'], listener['instance_port'],
+                      listener['elb_protocol'], instance_protocol]
+    if 'certificate' in listener:
+        listener_tuple.append(listener['certificate'])
+    return tuple(listener_tuple)
+
+
+def create(name, availability_zones, listeners, subnets=None,
            security_groups=None, scheme='internet-facing',
            region=None, key=None, keyid=None,
            profile=None):
     '''
     Create an ELB
 
-    CLI example to create an ELB::
+    CLI example to create an ELB:
 
-        salt myminion boto_elb.create myelb '["us-east-1a", "us-east-1e"]' listeners='[[443, 80, "HTTPS", "HTTP", "arn:aws:iam::1111111:server-certificate/mycert"]]' region=us-east-1
+    .. code-block:: bash
+
+        salt myminion boto_elb.create myelb '["us-east-1a", "us-east-1e"]' '{"elb_port": 443, "elb_protocol": "HTTPS", ...}' region=us-east-1
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
-
-    if __salt__['boto_elb.exists'](name, region, key, keyid, profile):
+    if exists(name, region, key, keyid, profile):
         return True
     if isinstance(availability_zones, string_types):
         availability_zones = json.loads(availability_zones)
+
     if isinstance(listeners, string_types):
         listeners = json.loads(listeners)
-    # Combining listeners and complex_listeners together makes our lives
-    # easier in some ways, especially since during introspection you can
-    # only get a combined set of listeners back from boto; however, boto
-    # requires us to send in separate listeners and complex listeners and
-    # the only real difference is the size. It feels like amazon/boto hate
-    # developers and wish to make us suffer.
-    _listeners = []
+
     _complex_listeners = []
     for listener in listeners:
-        if len(listener) <= 3:
-            _listeners.append(listener)
-        else:
-            _complex_listeners.append(listener)
+        _complex_listeners.append(listener_dict_to_tuple(listener))
+
     try:
-        lb = conn.create_load_balancer(name, availability_zones, _listeners,
-                                       subnets, security_groups, scheme,
-                                       _complex_listeners)
+        lb = conn.create_load_balancer(name=name, zones=availability_zones, subnets=subnets,
+                                       security_groups=security_groups, scheme=scheme,
+                                       complex_listeners=_complex_listeners)
         if lb:
             log.info('Created ELB {0}'.format(name))
             return True
@@ -204,13 +276,15 @@ def delete(name, region=None, key=None, keyid=None, profile=None):
     '''
     Delete an ELB.
 
-    CLI example to delete an ELB::
+    CLI example to delete an ELB:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.delete myelb region=us-east-1
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    if not __salt__['boto_elb.exists'](name, region, key, keyid, profile):
+    if not exists(name, region, key, keyid, profile):
         return True
     try:
         conn.delete_load_balancer(name)
@@ -224,35 +298,27 @@ def delete(name, region=None, key=None, keyid=None, profile=None):
         return False
 
 
-def create_listeners(name, listeners=None, region=None, key=None, keyid=None,
+def create_listeners(name, listeners, region=None, key=None, keyid=None,
                      profile=None):
     '''
     Create listeners on an ELB.
 
-    CLI example::
+    CLI example:
 
-        salt myminion boto_elb.create_listeners myelb listeners='[["HTTPS", "HTTP", 443, 80, "arn:aws:iam::11  11111:server-certificate/mycert"]]'
+    .. code-block:: bash
+
+        salt myminion boto_elb.create_listeners myelb '[["HTTPS", "HTTP", 443, 80, "arn:aws:iam::11  11111:server-certificate/mycert"]]'
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
     if isinstance(listeners, string_types):
         listeners = json.loads(listeners)
-    # Combining listeners and complex_listeners together makes our lives
-    # easier in some ways, especially since during introspection you can
-    # only get a combined set of listeners back from boto; however, boto
-    # requires us to send in separate listeners and complex listeners and
-    # the only real difference is the size. It feels like amazon/boto hate
-    # developers and wish to make us suffer.
-    _listeners = []
+
     _complex_listeners = []
     for listener in listeners:
-        if len(listener) <= 3:
-            _listeners.append(listener)
-        else:
-            _complex_listeners.append(listener)
+        _complex_listeners.append(listener_dict_to_tuple(listener))
     try:
-        conn.create_load_balancer_listeners(name, _listeners,
-                                            _complex_listeners)
+        conn.create_load_balancer_listeners(name, [], _complex_listeners)
         msg = 'Created ELB listeners on {0}'.format(name)
         log.info(msg)
         return True
@@ -268,7 +334,9 @@ def delete_listeners(name, ports, region=None, key=None, keyid=None,
     '''
     Delete listeners on an ELB.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.delete_listeners myelb '[80,443]'
     '''
@@ -293,7 +361,9 @@ def apply_security_groups(name, security_groups, region=None, key=None,
     '''
     Apply security groups to ELB.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.apply_security_groups myelb '["mysecgroup1"]'
     '''
@@ -319,7 +389,9 @@ def enable_availability_zones(name, availability_zones, region=None, key=None,
     '''
     Enable availability zones for ELB.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.enable_availability_zones myelb '["us-east-1a"]'
     '''
@@ -344,7 +416,9 @@ def disable_availability_zones(name, availability_zones, region=None, key=None,
     '''
     Disable availability zones for ELB.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.disable_availability_zones myelb '["us-east-1a"]'
     '''
@@ -369,7 +443,9 @@ def attach_subnets(name, subnets, region=None, key=None, keyid=None,
     '''
     Attach ELB to subnets.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.attach_subnets myelb '["mysubnet"]'
     '''
@@ -394,7 +470,9 @@ def detach_subnets(name, subnets, region=None, key=None, keyid=None,
     '''
     Detach ELB from subnets.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.detach_subnets myelb '["mysubnet"]'
     '''
@@ -418,7 +496,9 @@ def get_attributes(name, region=None, key=None, keyid=None, profile=None):
     '''
     Check to see if attributes are set on an ELB.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.get_attributes myelb
     '''
@@ -455,7 +535,40 @@ def set_attributes(name, attributes, region=None, key=None, keyid=None,
     '''
     Set attributes on an ELB.
 
-    CLI example to set attributes on an ELB::
+    name (string)
+        Name of the ELB instance to set attributes for
+
+    attributes
+        A dict of attributes to set.
+
+        Valid attributes are:
+
+        access_log (dict)
+            enabled (bool)
+                Enable storage of access logs.
+            s3_bucket_name (string)
+                The name of the S3 bucket to place logs.
+            s3_bucket_prefix (string)
+                Prefix for the log file name.
+            emit_interval (int)
+                Interval for storing logs in S3 in minutes. Valid values are
+                5 and 60.
+
+        connection_draining (dict)
+            enabled (bool)
+                Enable connection draining.
+            timeout (int)
+                Maximum allowed time in seconds for sending existing
+                connections to an instance that is deregistering or unhealthy.
+                Default is 300.
+
+        cross_zone_load_balancing (dict)
+            enabled (bool)
+                Enable cross-zone load balancing.
+
+    CLI example to set attributes on an ELB:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.set_attributes myelb '{"access_log": {"enabled": "true", "s3_bucket_name": "mybucket", "s3_bucket_prefix": "mylogs/", "emit_interval": "5"}}' region=us-east-1
     '''
@@ -524,7 +637,9 @@ def get_health_check(name, region=None, key=None, keyid=None, profile=None):
     '''
     Get the health check configured for this ELB.
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.get_health_check myelb
     '''
@@ -552,7 +667,9 @@ def set_health_check(name, health_check, region=None, key=None, keyid=None,
     '''
     Set attributes on an ELB.
 
-    CLI example to set attributes on an ELB::
+    CLI example to set attributes on an ELB:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.set_health_check myelb '{"target": "HTTP:80/"}'
     '''
@@ -580,7 +697,9 @@ def register_instances(name, instances, region=None, key=None, keyid=None,
     - ``True``: instance(s) registered successfully
     - ``False``: instance(s) failed to be registered
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.register_instances myelb instance_id
         salt myminion boto_elb.register_instances myelb "[instance_id,instance_id]"
@@ -594,7 +713,7 @@ def register_instances(name, instances, region=None, key=None, keyid=None,
     try:
         registered_instances = conn.register_instances(name, instances)
     except boto.exception.BotoServerError as error:
-        log.warn(error)
+        log.warning(error)
         return False
     registered_instance_ids = [instance.id for instance in
                                registered_instances]
@@ -602,7 +721,7 @@ def register_instances(name, instances, region=None, key=None, keyid=None,
     # able to be registered with the given ELB
     register_failures = set(instances).difference(set(registered_instance_ids))
     if register_failures:
-        log.warn('Instance(s): {0} not registered with ELB {1}.'
+        log.warning('Instance(s): {0} not registered with ELB {1}.'
                  .format(list(register_failures), name))
         register_result = False
     else:
@@ -622,7 +741,9 @@ def deregister_instances(name, instances, region=None, key=None, keyid=None,
     - ``False``: instance(s) failed to be deregistered
     - ``None``: instance(s) not valid or not registered, no action taken
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.deregister_instances myelb instance_id
         salt myminion boto_elb.deregister_instances myelb "[instance_id, instance_id]"
@@ -641,12 +762,12 @@ def deregister_instances(name, instances, region=None, key=None, keyid=None,
         # deregister_instances returns "None" because the instances are
         # effectively deregistered from ELB
         if error.error_code == 'InvalidInstance':
-            log.warn('One or more of instance(s) {0} are not part of ELB {1}.'
+            log.warning('One or more of instance(s) {0} are not part of ELB {1}.'
                      ' deregister_instances not performed.'
                      .format(instances, name))
             return None
         else:
-            log.warn(error)
+            log.warning(error)
             return False
     registered_instance_ids = [instance.id for instance in
                                registered_instances]
@@ -654,7 +775,7 @@ def deregister_instances(name, instances, region=None, key=None, keyid=None,
     # unable to be deregistered from the given ELB
     deregister_failures = set(instances).intersection(set(registered_instance_ids))
     if deregister_failures:
-        log.warn('Instance(s): {0} not deregistered from ELB {1}.'
+        log.warning('Instance(s): {0} not deregistered from ELB {1}.'
                  .format(list(deregister_failures), name))
         deregister_result = False
     else:
@@ -662,11 +783,40 @@ def deregister_instances(name, instances, region=None, key=None, keyid=None,
     return deregister_result
 
 
+def set_instances(name, instances, test=False, region=None, key=None, keyid=None,
+                         profile=None):
+    '''
+    Set the instances assigned to an ELB to exactly the list given
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.set_instances myelb region=us-east-1 instances="[instance_id,instance_id]"
+    '''
+    ret = True
+    current = set([i['instance_id'] for i in get_instance_health(name, region, key, keyid, profile)])
+    desired = set(instances)
+    add = desired - current
+    remove = current - desired
+    if test:
+        return bool(add or remove)
+    if len(remove):
+        if deregister_instances(name, list(remove), region, key, keyid, profile) is False:
+            ret = False
+    if len(add):
+        if register_instances(name, list(add), region, key, keyid, profile) is False:
+            ret = False
+    return ret
+
+
 def get_instance_health(name, region=None, key=None, keyid=None, profile=None, instances=None):
     '''
     Get a list of instances and their health state
 
-    CLI example::
+    CLI example:
+
+    .. code-block:: bash
 
         salt myminion boto_elb.get_instance_health myelb
         salt myminion boto_elb.get_instance_health myelb region=us-east-1 instances="[instance_id,instance_id]"
@@ -686,3 +836,249 @@ def get_instance_health(name, region=None, key=None, keyid=None, profile=None, i
     except boto.exception.BotoServerError as error:
         log.debug(error)
         return []
+
+
+def create_policy(name, policy_name, policy_type, policy, region=None,
+                  key=None, keyid=None, profile=None):
+    '''
+    Create an ELB policy.
+
+    .. versionadded:: 2016.3.0
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.create_policy myelb mypolicy LBCookieStickinessPolicyType '{"CookieExpirationPeriod": 3600}'
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if not exists(name, region, key, keyid, profile):
+        return False
+    try:
+        success = conn.create_lb_policy(name, policy_name, policy_type, policy)
+        if success:
+            log.info('Created policy {0} on ELB {1}'.format(policy_name, name))
+            return True
+        else:
+            msg = 'Failed to create policy {0} on ELB {1}'.format(policy_name, name)
+            log.error(msg)
+            return False
+    except boto.exception.BotoServerError as e:
+        log.debug(e)
+        msg = 'Failed to create policy {0} on ELB {1}: {2}'.format(policy_name, name, e.message)
+        log.error(msg)
+        return False
+
+
+def delete_policy(name, policy_name, region=None, key=None, keyid=None,
+                  profile=None):
+    '''
+    Delete an ELB policy.
+
+    .. versionadded:: 2016.3.0
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.delete_policy myelb mypolicy
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if not exists(name, region, key, keyid, profile):
+        return True
+    try:
+        conn.delete_lb_policy(name, policy_name)
+        log.info('Deleted policy {0} on ELB {1}'.format(policy_name, name))
+        return True
+    except boto.exception.BotoServerError as e:
+        log.debug(e)
+        msg = 'Failed to delete policy {0} on ELB {1}: {2}'.format(policy_name, name, e.message)
+        log.error(msg)
+        return False
+
+
+def set_listener_policy(name, port, policies=None, region=None, key=None,
+                        keyid=None, profile=None):
+    '''
+    Set the policies of an ELB listener.
+
+    .. versionadded:: 2016.3.0
+
+    CLI example:
+
+    .. code-block:: Bash
+
+        salt myminion boto_elb.set_listener_policy myelb 443 "[policy1,policy2]"
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if not exists(name, region, key, keyid, profile):
+        return True
+    if policies is None:
+        policies = []
+    try:
+        conn.set_lb_policies_of_listener(name, port, policies)
+        log.info('Set policies {0} on ELB {1} listener {2}'.format(policies, name, port))
+    except boto.exception.BotoServerError as e:
+        log.debug(e)
+        log.info('Failed to set policy {0} on ELB {1} listener {2}: {3}'.format(policies, name, port, e.message))
+        return False
+    return True
+
+
+def set_backend_policy(name, port, policies=None, region=None, key=None,
+                       keyid=None, profile=None):
+    '''
+    Set the policies of an ELB backend server.
+
+    CLI example:
+
+        salt myminion boto_elb.set_backend_policy myelb 443 "[policy1,policy2]"
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if not exists(name, region, key, keyid, profile):
+        return True
+    if policies is None:
+        policies = []
+    try:
+        conn.set_lb_policies_of_backend_server(name, port, policies)
+        log.info('Set policies {0} on ELB {1} backend server {2}'.format(policies, name, port))
+    except boto.exception.BotoServerError as e:
+        log.debug(e)
+        log.info('Failed to set policy {0} on ELB {1} backend server {2}: {3}'.format(policies, name, port, e.message))
+        return False
+    return True
+
+
+def set_tags(name, tags, region=None, key=None, keyid=None, profile=None):
+    '''
+    Add the tags on an ELB
+
+    .. versionadded:: 2016.3.0
+
+    name
+        name of the ELB
+
+    tags
+        dict of name/value pair tags
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.set_tags my-elb-name "{'Tag1': 'Value', 'Tag2': 'Another Value'}"
+    '''
+
+    if exists(name, region, key, keyid, profile):
+        conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+        ret = _add_tags(conn, name, tags)
+        return ret
+    else:
+        return False
+
+
+def delete_tags(name, tags, region=None, key=None, keyid=None, profile=None):
+    '''
+    Add the tags on an ELB
+
+    name
+        name of the ELB
+
+    tags
+        list of tags to remove
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion boto_elb.delete_tags my-elb-name ['TagToRemove1', 'TagToRemove2']
+    '''
+    if exists(name, region, key, keyid, profile):
+        conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+        ret = _remove_tags(conn, name, tags)
+        return ret
+    else:
+        return False
+
+
+def _build_tag_param_list(params, tags):
+    '''
+    helper function to build a tag parameter list to send
+    '''
+    keys = sorted(tags.keys())
+    i = 1
+    for key in keys:
+        value = tags[key]
+        params['Tags.member.{0}.Key'.format(i)] = key
+        if value is not None:
+            params['Tags.member.{0}.Value'.format(i)] = value
+        i += 1
+
+
+def _get_all_tags(conn, load_balancer_names=None):
+    '''
+    Retrieve all the metadata tags associated with your ELB(s).
+
+    :type load_balancer_names: list
+    :param load_balancer_names: An optional list of load balancer names.
+
+    :rtype: list
+    :return: A list of :class:`boto.ec2.elb.tag.Tag` objects
+    '''
+    params = {}
+    if load_balancer_names:
+        conn.build_list_params(params, load_balancer_names,
+                               'LoadBalancerNames.member.%d')
+
+    tags = conn.get_object(
+        'DescribeTags',
+        params,
+        __utils__['boto_elb_tag.get_tag_descriptions'](),
+        verb='POST'
+    )
+    if tags[load_balancer_names]:
+        return tags[load_balancer_names]
+    else:
+        return None
+
+
+def _add_tags(conn, load_balancer_names, tags):
+    '''
+    Create new metadata tags for the specified resource ids.
+
+    :type load_balancer_names: list
+    :param load_balancer_names: A list of load balancer names.
+
+    :type tags: dict
+    :param tags: A dictionary containing the name/value pairs.
+                 If you want to create only a tag name, the
+                 value for that tag should be the empty string
+                 (e.g. '').
+    '''
+    params = {}
+    conn.build_list_params(params, load_balancer_names,
+                           'LoadBalancerNames.member.%d')
+    _build_tag_param_list(params, tags)
+    return conn.get_status('AddTags', params, verb='POST')
+
+
+def _remove_tags(conn, load_balancer_names, tags):
+    '''
+    Delete metadata tags for the specified resource ids.
+
+    :type load_balancer_names: list
+    :param load_balancer_names: A list of load balancer names.
+
+    :type tags: list
+    :param tags: A list containing just tag names for the tags to be
+                 deleted.
+    '''
+    params = {}
+    conn.build_list_params(params, load_balancer_names,
+                           'LoadBalancerNames.member.%d')
+    conn.build_list_params(params, tags,
+                           'Tags.member.%d.Key')
+    return conn.get_status('RemoveTags', params, verb='POST')
