@@ -5,7 +5,7 @@ Connection module for Amazon Security Groups
 .. versionadded:: 2014.7.0
 
 :configuration: This module accepts explicit ec2 credentials but can
-    also utilize IAM roles assigned to the instance trough Instance Profiles.
+    also utilize IAM roles assigned to the instance through Instance Profiles.
     Dynamic credentials are then automatically obtained from AWS API and no
     further configuration is necessary. More Information available at:
 
@@ -243,6 +243,68 @@ def _parse_rules(sg, rules):
     return _rules
 
 
+def get_all_security_groups(groupnames=None, group_ids=None, filters=None,
+                            region=None, key=None, keyid=None, profile=None):
+    '''
+    Return a list of all Security Groups matching the given criteria and filters.
+
+    Note that the 'groupnames' argument only functions correctly for EC2 Classic
+    and default VPC Security Groups.  To find groups by name in other VPCs you'll
+    want to use the 'group-name' filter instead.
+
+    Valid keys for the filters argument are:
+        description - The description of the security group.
+        egress.ip-permission.prefix-list-id - The ID (prefix) of the AWS service to which the security group allows access.
+        group-id - The ID of the security group.
+        group-name - The name of the security group.
+        ip-permission.cidr - A CIDR range that has been granted permission.
+        ip-permission.from-port - The start of port range for the TCP and UDP protocols, or an ICMP type number.
+        ip-permission.group-id - The ID of a security group that has been granted permission.
+        ip-permission.group-name - The name of a security group that has been granted permission.
+        ip-permission.protocol - The IP protocol for the permission (tcp | udp | icmp or a protocol number).
+        ip-permission.to-port - The end of port range for the TCP and UDP protocols, or an ICMP code.
+        ip-permission.user-id - The ID of an AWS account that has been granted permission.
+        owner-id - The AWS account ID of the owner of the security group.
+        tag-key - The key of a tag assigned to the security group.
+        tag-value - The value of a tag assigned to the security group.
+        vpc-id - The ID of the VPC specified when the security group was created.
+
+    CLI example::
+
+        salt myminion boto_secgroup.get_all_security_groups filters='{group-name: mygroup}'
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if isinstance(groupnames, str):
+        groupnames = [groupnames]
+    if isinstance(group_ids, str):
+        groupnames = [group_ids]
+
+    interesting = ['description', 'id', 'instances', 'name', 'owner_id',
+                   'region', 'rules', 'rules_egress', 'tags', 'vpc_id']
+    ret = []
+    try:
+        r = conn.get_all_security_groups(groupnames=groupnames,
+                                         group_ids=group_ids,
+                                         filters=filters)
+        for g in r:
+            n = {}
+            for a in interesting:
+                v = getattr(g, a, None)
+                if a == 'region':
+                    v = v.name
+                elif a in ('rules', 'rules_egress'):
+                    v = _parse_rules(g, v)
+                elif a == 'instances':
+                    v = [i.id for i in v()]
+                n[a] = v
+            ret += [n]
+        return ret
+    except boto.exception.BotoServerError as e:
+        log.debug(e)
+        return []
+
+
 def get_group_id(name, vpc_id=None, vpc_name=None, region=None, key=None,
                  keyid=None, profile=None):
     '''
@@ -262,7 +324,7 @@ def get_group_id(name, vpc_id=None, vpc_name=None, region=None, key=None,
         return False
 
 
-def convert_to_group_ids(groups, vpc_id, vpc_name=None, region=None, key=None,
+def convert_to_group_ids(groups, vpc_id=None, vpc_name=None, region=None, key=None,
                          keyid=None, profile=None):
     '''
     Given a list of security groups and a vpc_id, convert_to_group_ids will
@@ -276,8 +338,7 @@ def convert_to_group_ids(groups, vpc_id, vpc_name=None, region=None, key=None,
     group_ids = []
     for group in groups:
         if re.match('sg-.*', group):
-            log.debug('group {0} is a group id. get_group_id not called.'
-                      .format(group))
+            log.debug('group {0} is a group id. get_group_id not called.'.format(group))
             group_ids.append(group)
         else:
             log.debug('calling boto_secgroup.get_group_id for'
@@ -285,10 +346,12 @@ def convert_to_group_ids(groups, vpc_id, vpc_name=None, region=None, key=None,
             group_id = get_group_id(name=group, vpc_id=vpc_id,
                                     vpc_name=vpc_name, region=region,
                                     key=key, keyid=keyid, profile=profile)
-            log.debug('group name {0} has group id {1}'.format(
-                group, group_id)
-            )
-            group_ids.append(str(group_id))
+            if not group_id:
+                log.warning('group name {0} did not resolve to a group ID'.format(group))
+            else:
+                log.debug('group name {0} has group id {1}'.format(group, group_id))
+                group_ids.append(str(group_id))
+
     log.debug('security group contents {0} post-conversion'.format(group_ids))
     return group_ids
 
