@@ -199,63 +199,59 @@ def set_channel_access(channel=14, access_update_mode='non_volatile',
         - 'non_volatile' = set non-volatile Channel Access
         - 'volatile'     = set volatile (active) setting of Channel Access
 
-    :param alerting: PEF Alerting Enable/Disable
+    :param alerting:
+        PEF Alerting Enable/Disable
+            - True  = enable PEF Alerting
+            - False = disable PEF Alerting on this channel
+            (Alert Immediate command can still be used to generate alerts)
 
-        - True  = enable PEF Alerting
-        - False = disable PEF Alerting on this channel
-          (Alert Immediate command can still be used to generate alerts)
+    :param per_msg_auth:
+        Per-message Authentication
+            - True  = enable
+            - False = disable Per-message Authentication. [Authentication required to
+              activate any session on this channel, but authentication not
+              used on subsequent packets for the session.]
 
-    :param per_msg_auth: Per-message Authentication
-
-        - True  = enable
-        - False = disable Per-message Authentication. [Authentication required to
-          activate any session on this channel, but authentication not used on
-          subsequent packets for the session.]
-
-    :param user_level_auth: User Level Authentication Enable/Disable.
-
-        - True  = enable User Level Authentication. All User Level commands are
-          to be authenticated per the Authentication Type that was
-          negotiated when the session was activated.
-        - False = disable User Level Authentication. Allow User Level commands to
-          be executed without being authenticated.
-
-          If the option to disable User Level Command authentication is
-          accepted, the BMC will accept packets with Authentication Type
-          set to None if they contain user level commands.
-
-          For outgoing packets, the BMC returns responses with the same
-          Authentication Type that was used for the request.
-
-    :param access_mode: Access Mode for IPMI messaging
+    :param user_level_auth:
+        User Level Authentication Enable/Disable.
+            - True  = enable User Level Authentication. All User Level commands are
+              to be authenticated per the Authentication Type that was
+              negotiated when the session was activated.
+            - False = disable User Level Authentication. Allow User Level commands to
+              be executed without being authenticated.
+              If the option to disable User Level Command authentication is
+              accepted, the BMC will accept packets with Authentication Type
+              set to None if they contain user level commands.
+              For outgoing packets, the BMC returns responses with the same
+              Authentication Type that was used for the request.
 
+    :param access_mode:
+        Access Mode for IPMI messaging
         (PEF Alerting is enabled/disabled separately from IPMI messaging)
+            - disabled = disabled for IPMI messaging
+            - pre_boot = pre-boot only channel only available when system is
+              in a powered down state or in BIOS prior to start of boot.
+            - always   = channel always available regardless of system mode.
+              BIOS typically dedicates the serial connection to the BMC.
+            - shared   = same as always available, but BIOS typically leaves the
+              serial port available for software use.
 
-        * disabled = disabled for IPMI messaging
-        * pre_boot = pre-boot only channel only available when system is in a
-          powered down state or in BIOS prior to start of boot.
-        * always   = channel always available regardless of system mode.
-          BIOS typically dedicates the serial connection to the BMC.
-        * shared   = same as always available, but BIOS typically leaves the
-          serial port available for software use.
+    :param privilege_update_mode:
+        Channel Privilege Level Limit.
+        This value sets the maximum privilege level
+        that can be accepted on the specified channel.
+            - dont_change  = don't set or change channel Privilege Level Limit
+            - non_volatile = non-volatile Privilege Level Limit according
+            - volatile     = volatile setting of Privilege Level Limit
 
-    :param privilege_update_mode: Channel Privilege Level Limit.
-
-        This value sets the maximum privilege level that can be accepted on the
-        specified channel.
-
-        * dont_change  = don't set or change channel Privilege Level Limit
-        * non_volatile = non-volatile Privilege Level Limit according
-        * volatile     = volatile setting of Privilege Level Limit
-
-    :param privilege_level: Channel Privilege Level Limit
-
-        * reserved      = unused
-        * callback
-        * user
-        * operator
-        * administrator
-        * proprietary   = used by OEM
+    :param privilege_level:
+        Channel Privilege Level Limit
+            - reserved      = unused
+            - callback
+            - user
+            - operator
+            - administrator
+            - proprietary   = used by OEM
 
     :param kwargs:
         - api_host=127.0.0.1
@@ -367,16 +363,16 @@ def set_user_access(uid, channel=14, callback=True, link_auth=True, ipmi_msg=Tru
 
     :param channel: number [1:7]
 
-    :parm callback: User Restricted to Callback
-
-        - False = User Privilege Limit is determined by the User Privilege Limit
-          parameter, below, for both callback and non-callback connections.
-        - True  = User Privilege Limit is determined by the User Privilege Limit
-          parameter for callback connections, but is restricted to Callback
-          level for non-callback connections. Thus, a user can only initiate
-          a Callback when they 'call in' to the BMC, but once the callback
-          connection has been made, the user could potentially establish a
-          session as an Operator.
+    :param callback:
+        User Restricted to Callback
+            - False = User Privilege Limit is determined by the User Privilege Limit
+              parameter, below, for both callback and non-callback connections.
+            - True  = User Privilege Limit is determined by the User Privilege Limit
+              parameter for callback connections, but is restricted to Callback
+              level for non-callback connections. Thus, a user can only initiate
+              a Callback when they 'call in' to the BMC, but once the callback
+              connection has been made, the user could potentially establish a
+              session as an Operator.
 
     :param link_auth: User Link authentication
         enable/disable (used to enable whether this
@@ -386,26 +382,26 @@ def set_user_access(uid, channel=14, callback=True, link_auth=True, ipmi_msg=Tru
         enabled/disabled via the serial/modem configuration parameters.
 
     :param ipmi_msg: User IPMI Messaging:
-        (used to enable/disable whether this user's name and password
-        information will be used for IPMI Messaging. In this case, 'IPMI
-        Messaging' refers to the ability to execute generic IPMI commands that
-        are not associated with a particular payload type. For example, if IPMI
-        Messaging is disabled for a user, but that user is enabled for
-        activating the SOL payload type, then IPMI commands associated with SOL
-        and session management, such as Get SOL Configuration Parameters and
-        Close Session are available, but generic IPMI commands such as Get SEL
-        Time are unavailable.)
+        (used to enable/disable whether
+        this user's name and password information will be used for IPMI
+        Messaging. In this case, 'IPMI Messaging' refers to the ability to
+        execute generic IPMI commands that are not associated with a
+        particular payload type. For example, if IPMI Messaging is disabled for
+        a user, but that user is enabled for activating the SOL
+        payload type, then IPMI commands associated with SOL and session
+        management, such as Get SOL Configuration Parameters and Close Session
+        are available, but generic IPMI commands such as Get SEL Time are
+        unavailable.)
 
-    :param privilege_level: User Privilege Limit.
-        (Determines the maximum privilege level that the user is allowed to
-        switch to on the specified channel.)
-
-        - callback
-        - user
-        - operator
-        - administrator
-        - proprietary
-        - no_access
+    :param privilege_level:
+        User Privilege Limit. (Determines the maximum privilege level that the
+        user is allowed to switch to on the specified channel.)
+            - callback
+            - user
+            - operator
+            - administrator
+            - proprietary
+            - no_access
 
     :param kwargs:
         - api_host=127.0.0.1
@@ -708,7 +704,7 @@ def set_bootdev(bootdev='default', persist=False, uefiboot=False, **kwargs):
                     this
 
     :param uefiboot: If true, request UEFI boot explicitly.  Strictly
-                    speaking, the spec sugests that if not set, the system
+                    speaking, the spec suggests that if not set, the system
                     should BIOS boot and offers no "don't care" option.
                     In practice, this flag not being set does not preclude
                     UEFI boot on any system I've encountered.
