@@ -22,11 +22,14 @@ __func_alias__ = {
     'time_': 'time',
 }
 
+__virtualname__ = 'osquery'
+
 
 def __virtual__():
     if salt.utils.which('osqueryi'):
-        return 'osquery'
-    return False
+        return __virtualname__
+    return (False, 'The osquery execution module cannot be loaded: '
+                   'osqueryi binary is not in the path.')
 
 
 def _table_attrs(table):
@@ -656,7 +659,7 @@ def alf(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='alf', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def alf_exceptions(attrs=None, where=None):
@@ -671,7 +674,7 @@ def alf_exceptions(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='alf_exceptions', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def alf_explicit_auths(attrs=None, where=None):
@@ -686,7 +689,7 @@ def alf_explicit_auths(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='alf_explicit_auths', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def alf_services(attrs=None, where=None):
@@ -701,7 +704,7 @@ def alf_services(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='alf_services', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def apps(attrs=None, where=None):
@@ -716,7 +719,7 @@ def apps(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='apps', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def certificates(attrs=None, where=None):
@@ -731,7 +734,7 @@ def certificates(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='certificates', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def chrome_extensions(attrs=None, where=None):
@@ -746,7 +749,7 @@ def chrome_extensions(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='chrome_extensions', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def firefox_addons(attrs=None, where=None):
@@ -761,7 +764,7 @@ def firefox_addons(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='firefox_addons', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def homebrew_packages(attrs=None, where=None):
@@ -776,7 +779,7 @@ def homebrew_packages(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='homebrew_packages', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def iokit_devicetree(attrs=None, where=None):
@@ -791,7 +794,7 @@ def iokit_devicetree(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='iokit_devicetree', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def iokit_registry(attrs=None, where=None):
@@ -806,7 +809,7 @@ def iokit_registry(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='iokit_registry', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def kernel_extensions(attrs=None, where=None):
@@ -821,7 +824,7 @@ def kernel_extensions(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='kernel_extensions', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def keychain_items(attrs=None, where=None):
@@ -836,7 +839,7 @@ def keychain_items(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='keychain_items', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def launchd(attrs=None, where=None):
@@ -851,7 +854,7 @@ def launchd(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='launchd', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def nfs_shares(attrs=None, where=None):
@@ -866,7 +869,7 @@ def nfs_shares(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='nfs_shares', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def nvram(attrs=None, where=None):
@@ -881,7 +884,7 @@ def nvram(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='nvram', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def preferences(attrs=None, where=None):
@@ -896,7 +899,7 @@ def preferences(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='preferences', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def quarantine(attrs=None, where=None):
@@ -911,7 +914,7 @@ def quarantine(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='quarantine', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def safari_extensions(attrs=None, where=None):
@@ -926,7 +929,7 @@ def safari_extensions(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='safari_extensions', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def startup_items(attrs=None, where=None):
@@ -941,7 +944,7 @@ def startup_items(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='startup_items', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def xattr_where_from(attrs=None, where=None):
@@ -956,7 +959,7 @@ def xattr_where_from(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='xattr_where_from', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def xprotect_entries(attrs=None, where=None):
@@ -971,7 +974,7 @@ def xprotect_entries(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='xprotect_entries', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def xprotect_reports(attrs=None, where=None):
@@ -986,7 +989,7 @@ def xprotect_reports(attrs=None, where=None):
     '''
     if salt.utils.is_darwin():
         return _osquery_cmd(table='xprotect_reports', attrs=attrs, where=where)
-    return {'result': False, 'comment': 'Only available on OS X systems.'}
+    return {'result': False, 'comment': 'Only available on macOS systems.'}
 
 
 def file_(attrs=None, where=None):
