@@ -79,14 +79,16 @@ from __future__ import absolute_import
 # Import python libs
 import os
 import re
-import logging
 import shutil
+import logging
+import sys
 
 # Import salt libs
 import salt.utils
+import tempfile
 import salt.utils.locales
 import salt.utils.url
-from salt.ext.six import string_types
+from salt.ext import six
 from salt.exceptions import CommandExecutionError, CommandNotFoundError
 
 
@@ -99,6 +101,8 @@ __func_alias__ = {
 
 VALID_PROTOS = ['http', 'https', 'ftp', 'file']
 
+rex_pip_chain_read = re.compile(r'-r\s(.*)\n?', re.MULTILINE)
+
 
 def __virtual__():
     '''
@@ -115,7 +119,10 @@ def _get_pip_bin(bin_env):
     executable itself, or from searching conventional filesystem locations
     '''
     if not bin_env:
-        which_result = __salt__['cmd.which_bin'](['pip2', 'pip', 'pip-python'])
+        which_result = __salt__['cmd.which_bin'](
+            ['pip{0}.{1}'.format(*sys.version_info[:2]),
+             'pip', 'pip2', 'pip3', 'pip-python']
+        )
         if which_result is None:
             raise CommandNotFoundError('Could not find a `pip` binary')
         if salt.utils.is_windows():
@@ -125,7 +132,8 @@ def _get_pip_bin(bin_env):
     # try to get pip bin from virtualenv, bin_env
     if os.path.isdir(bin_env):
         if salt.utils.is_windows():
-            pip_bin = os.path.join(bin_env, 'Scripts', 'pip.exe').encode('string-escape')
+            pip_bin = os.path.join(
+                bin_env, 'Scripts', 'pip.exe').encode('string-escape')
         else:
             pip_bin = os.path.join(bin_env, 'bin', 'pip')
         if os.path.isfile(pip_bin):
@@ -188,17 +196,53 @@ def _get_env_activate(bin_env):
     raise CommandNotFoundError('Could not find a `activate` binary')
 
 
-def _process_requirements(requirements, cmd, saltenv, user, no_chown):
+def _find_req(link):
+
+    logger.info('_find_req -- link = %s', str(link))
+
+    with salt.utils.fopen(link) as fh_link:
+        child_links = rex_pip_chain_read.findall(fh_link.read())
+
+    base_path = os.path.dirname(link)
+    child_links = [os.path.join(base_path, d) for d in child_links]
+
+    return child_links
+
+
+def _resolve_requirements_chain(requirements):
+    '''
+    Return an array of requirements file paths that can be used to complete
+    the no_chown==False && user != None conundrum
+    '''
+
+    chain = []
+
+    if isinstance(requirements, six.string_types):
+        requirements = [requirements]
+
+    for req_file in requirements:
+        chain.append(req_file)
+        chain.extend(_resolve_requirements_chain(_find_req(req_file)))
+
+    return chain
+
+
+def _process_requirements(requirements, cmd, cwd, saltenv, user):
     '''
     Process the requirements argument
     '''
     cleanup_requirements = []
+
     if requirements is not None:
-        if isinstance(requirements, string_types):
+        if isinstance(requirements, six.string_types):
             requirements = [r.strip() for r in requirements.split(',')]
+        elif not isinstance(requirements, list):
+            raise TypeError('requirements must be a string or list')
+
+        treq = None
 
         for requirement in requirements:
-            treq = None
+            logger.debug('TREQ IS: %s', str(treq))
             if requirement.startswith('salt://'):
                 cached_requirements = _get_cached_requirements(
                     requirement, saltenv
@@ -210,24 +254,84 @@ def _process_requirements(requirements, cmd, saltenv, user, no_chown):
                     return None, ret
                 requirement = cached_requirements
 
-            if user and not no_chown:
+            if user:
                 # Need to make a temporary copy since the user will, most
                 # likely, not have the right permissions to read the file
-                treq = salt.utils.mkstemp()
-                shutil.copyfile(requirement, treq)
-                logger.debug(
-                    'Changing ownership of requirements file \'{0}\' to '
-                    'user \'{1}\''.format(treq, user)
-                )
+
+                if not treq:
+                    treq = tempfile.mkdtemp()
+
                 __salt__['file.chown'](treq, user, None)
-                cleanup_requirements.append(treq)
-            cmd.extend(['--requirement', treq or requirement])
+
+                current_directory = None
+
+                if not current_directory:
+                    current_directory = os.path.abspath(os.curdir)
+
+                logger.info('_process_requirements from directory,' +
+                            '%s -- requirement: %s', cwd, requirement
+                            )
+
+                if cwd is None:
+                    r = requirement
+                    c = cwd
+
+                    requirement_abspath = os.path.abspath(requirement)
+                    cwd = os.path.dirname(requirement_abspath)
+                    requirement = os.path.basename(requirement)
+
+                    logger.debug('\n\tcwd: %s -> %s\n\trequirement: %s -> %s\n',
+                                 c, cwd, r, requirement
+                                 )
+
+                os.chdir(cwd)
+
+                reqs = _resolve_requirements_chain(requirement)
+
+                os.chdir(current_directory)
+
+                logger.info('request files: {0}'.format(str(reqs)))
+
+                for req_file in reqs:
+                    if not os.path.isabs(req_file):
+                        req_file = os.path.join(cwd, req_file)
+
+                    logger.debug('TREQ N CWD: %s -- %s -- for %s', str(treq), str(cwd), str(req_file))
+                    target_path = os.path.join(treq, os.path.basename(req_file))
+
+                    logger.debug('S: %s', req_file)
+                    logger.debug('T: %s', target_path)
+
+                    target_base = os.path.dirname(target_path)
+
+                    if not os.path.exists(target_base):
+                        os.makedirs(target_base, mode=0o755)
+                        __salt__['file.chown'](target_base, user, None)
+
+                    if not os.path.exists(target_path):
+                        logger.debug(
+                            'Copying %s to %s', req_file, target_path
+                        )
+                        __salt__['file.copy'](req_file, target_path)
+
+                    logger.debug(
+                        'Changing ownership of requirements file \'{0}\' to '
+                        'user \'{1}\''.format(target_path, user)
+                    )
+
+                    __salt__['file.chown'](target_path, user, None)
+
+            req_args = os.path.join(treq, requirement) if treq else requirement
+            cmd.extend(['--requirement', req_args])
+
+        cleanup_requirements.append(treq)
+
+    logger.debug('CLEANUP_REQUIREMENTS: %s', str(cleanup_requirements))
     return cleanup_requirements, None
 
 
 def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
             requirements=None,
-            env=None,
             bin_env=None,
             use_wheel=False,
             no_use_wheel=False,
@@ -257,18 +361,18 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
             user=None,
             no_chown=False,
             cwd=None,
-            activate=False,
             pre_releases=False,
             cert=None,
             allow_all_external=False,
             allow_external=None,
             allow_unverified=None,
             process_dependency_links=False,
-            __env__=None,
             saltenv='base',
             env_vars=None,
             use_vt=False,
-            trusted_host=None):
+            trusted_host=None,
+            no_cache_dir=False,
+            cache_dir=None):
     '''
     Install packages with pip
 
@@ -290,9 +394,6 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
             If installing into a virtualenv, just use the path to the
             virtualenv (e.g. ``/home/code/path/to/virtualenv/``)
 
-    env
-        Deprecated, use bin_env now
-
     use_wheel
         Prefer wheel archives (requires pip>=1.4)
 
@@ -330,6 +431,11 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     mirrors
         Specific mirror URL(s) to query (automatically adds --use-mirrors)
 
+        .. warning::
+
+            This option has been deprecated and removed in pip version 7.0.0.
+            Please use ``index_url`` and/or ``extra_index_url`` instead.
+
     build
         Unpack packages into ``build`` dir
 
@@ -339,8 +445,8 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     download
         Download packages into ``download`` instead of installing them
 
-    download_cache
-        Cache downloaded packages in ``download_cache`` dir
+    download_cache | cache_dir
+        Cache downloaded packages in ``download_cache`` or ``cache_dir`` dir
 
     source
         Check out ``editable`` packages into ``source`` dir
@@ -390,14 +496,6 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     cwd
         Current working directory to run pip from
 
-    activate
-        Activates the virtual environment, if given via bin_env, before running
-        install.
-
-        .. deprecated:: 2014.7.2
-            If `bin_env` is given, pip will already be sourced from that
-            virualenv, making `activate` effectively a noop.
-
     pre_releases
         Include pre-releases in the available versions
 
@@ -435,7 +533,10 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         HTTPS.
 
     use_vt
-        Use VT terminal emulation (see ouptut while installing)
+        Use VT terminal emulation (see output while installing)
+
+    no_cache_dir
+        Disable the cache.
 
     CLI Example:
 
@@ -452,38 +553,6 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
                 editable=git+https://github.com/worldcompany/djangoembed.git#egg=djangoembed upgrade=True no_deps=True
 
     '''
-    # Switching from using `pip_bin` and `env` to just `bin_env`
-    # cause using an env and a pip bin that's not in the env could
-    # be problematic.
-    # Still using the `env` variable, for backwards compatibility's sake
-    # but going fwd you should specify either a pip bin or an env with
-    # the `bin_env` argument and we'll take care of the rest.
-    if env and not bin_env:
-        salt.utils.warn_until(
-            'Boron',
-            'Passing \'env\' to the pip module is deprecated. Use bin_env instead. '
-            'This functionality will be removed in Salt Boron.'
-        )
-        bin_env = env
-
-    if activate:
-        salt.utils.warn_until(
-                'Boron',
-                'Passing \'activate\' to the pip module is deprecated. If '
-                'bin_env refers to a virtualenv, there is no need to activate '
-                'that virtualenv before using pip to install packages in it.'
-        )
-
-    if isinstance(__env__, string_types):
-        salt.utils.warn_until(
-            'Boron',
-            'Passing a salt environment should be done using \'saltenv\' '
-            'not \'__env__\'. This functionality will be removed in Salt '
-            'Boron.'
-        )
-        # Backwards compatibility
-        saltenv = __env__
-
     pip_bin = _get_pip_bin(bin_env)
 
     cmd = [pip_bin, 'install']
@@ -491,9 +560,10 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     cleanup_requirements, error = _process_requirements(
         requirements=requirements,
         cmd=cmd,
+        cwd=cwd,
         saltenv=saltenv,
-        user=user,
-        no_chown=no_chown)
+        user=user
+    )
 
     if error:
         return error
@@ -503,7 +573,7 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         cur_version = __salt__['pip.version'](bin_env)
         if not salt.utils.compare_versions(ver1=cur_version, oper='>=',
                                            ver2=min_version):
-            log.error(
+            logger.error(
                 ('The --use-wheel option is only supported in pip {0} and '
                  'newer. The version of pip detected is {1}. This option '
                  'will be ignored.'.format(min_version, cur_version))
@@ -516,7 +586,7 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         cur_version = __salt__['pip.version'](bin_env)
         if not salt.utils.compare_versions(ver1=cur_version, oper='>=',
                                            ver2=min_version):
-            log.error(
+            logger.error(
                 ('The --no-use-wheel option is only supported in pip {0} and '
                  'newer. The version of pip detected is {1}. This option '
                  'will be ignored.'.format(min_version, cur_version))
@@ -525,10 +595,10 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
             cmd.append('--no-use-wheel')
 
     if log:
-        try:
-            # TODO make this check if writeable
-            os.path.exists(log)
-        except IOError:
+        if os.path.isdir(log):
+            raise IOError(
+                '\'{0}\' is a directory. Use --log path_to_file'.format(log))
+        elif not os.access(log, os.W_OK):
             raise IOError('\'{0}\' is not writeable'.format(log))
 
         cmd.extend(['--log', log])
@@ -551,7 +621,7 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         cmd.extend(['--timeout', timeout])
 
     if find_links:
-        if isinstance(find_links, string_types):
+        if isinstance(find_links, six.string_types):
             find_links = [l.strip() for l in find_links.split(',')]
 
         for link in find_links:
@@ -585,7 +655,15 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         cmd.append('--no-index')
 
     if mirrors:
-        if isinstance(mirrors, string_types):
+        # https://github.com/pypa/pip/pull/2641/files#diff-3ef137fb9ffdd400f117a565cd94c188L216
+        pip_version = version(pip_bin)
+        if salt.utils.compare_versions(ver1=pip_version, oper='>=', ver2='7.0.0'):
+            raise CommandExecutionError(
+                    'pip >= 7.0.0 does not support mirror argument:'
+                    ' use index_url and/or extra_index_url instead'
+            )
+
+        if isinstance(mirrors, six.string_types):
             mirrors = [m.strip() for m in mirrors.split(',')]
 
         cmd.append('--use-mirrors')
@@ -605,8 +683,10 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     if download:
         cmd.extend(['--download', download])
 
-    if download_cache:
-        cmd.extend(['--download-cache', download_cache])
+    if download_cache or cache_dir:
+        cmd.extend(['--cache-dir' if salt.utils.compare_versions(
+            ver1=version(bin_env), oper='>=', ver2='6.0'
+        ) else '--download-cache', download_cache or cache_dir])
 
     if source:
         cmd.extend(['--source', source])
@@ -637,6 +717,9 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     if no_download:
         cmd.append('--no-download')
 
+    if no_cache_dir:
+        cmd.append('--no-cache-dir')
+
     if pre_releases:
         # Check the locally installed pip version
         pip_version = version(pip_bin)
@@ -649,21 +732,21 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         cmd.extend(['--cert', cert])
 
     if global_options:
-        if isinstance(global_options, string_types):
+        if isinstance(global_options, six.string_types):
             global_options = [go.strip() for go in global_options.split(',')]
 
         for opt in global_options:
             cmd.extend(['--global-option', opt])
 
     if install_options:
-        if isinstance(install_options, string_types):
+        if isinstance(install_options, six.string_types):
             install_options = [io.strip() for io in install_options.split(',')]
 
         for opt in install_options:
             cmd.extend(['--install-option', opt])
 
     if pkgs:
-        if isinstance(pkgs, string_types):
+        if isinstance(pkgs, six.string_types):
             pkgs = [p.strip() for p in pkgs.split(',')]
 
         # It's possible we replaced version-range commas with semicolons so
@@ -674,7 +757,7 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
 
     if editable:
         egg_match = re.compile(r'(?:#|#.*?&)egg=([^&]*)')
-        if isinstance(editable, string_types):
+        if isinstance(editable, six.string_types):
             editable = [e.strip() for e in editable.split(',')]
 
         for entry in editable:
@@ -693,14 +776,14 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         cmd.append('--allow-all-external')
 
     if allow_external:
-        if isinstance(allow_external, string_types):
+        if isinstance(allow_external, six.string_types):
             allow_external = [p.strip() for p in allow_external.split(',')]
 
         for pkg in allow_external:
             cmd.extend(['--allow-external', pkg])
 
     if allow_unverified:
-        if isinstance(allow_unverified, string_types):
+        if isinstance(allow_unverified, six.string_types):
             allow_unverified = \
                 [p.strip() for p in allow_unverified.split(',')]
 
@@ -710,28 +793,40 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
     if process_dependency_links:
         cmd.append('--process-dependency-links')
 
-    if env_vars:
-        if isinstance(env_vars, dict):
-            os.environ.update(env_vars)
-        else:
-            raise CommandExecutionError('env_vars {0} is not a dictionary'.format(env_vars))
-
     if trusted_host:
         cmd.extend(['--trusted-host', trusted_host])
 
+    cmd_kwargs = dict(saltenv=saltenv, use_vt=use_vt, runas=user)
+
+    if env_vars:
+        if isinstance(env_vars, dict):
+            for key, val in six.iteritems(env_vars):
+                if not isinstance(val, six.string_types):
+                    val = str(val)
+                cmd_kwargs.setdefault('env', {})[key] = val
+        else:
+            raise CommandExecutionError(
+                'env_vars {0} is not a dictionary'.format(env_vars))
+
     try:
-        cmd_kwargs = dict(cwd=cwd, saltenv=saltenv, use_vt=use_vt, runas=user)
+        if cwd:
+            cmd_kwargs['cwd'] = cwd
+
         if bin_env and os.path.isdir(bin_env):
-            cmd_kwargs['env'] = {'VIRTUAL_ENV': bin_env}
+            cmd_kwargs.setdefault('env', {})['VIRTUAL_ENV'] = bin_env
+
+        logger.debug(
+            'TRY BLOCK: end of pip.install -- cmd: %s, cmd_kwargs: %s',
+            str(cmd), str(cmd_kwargs)
+        )
+
         return __salt__['cmd.run_all'](cmd,
                                        python_shell=False,
                                        **cmd_kwargs)
     finally:
-        for requirement in cleanup_requirements:
-            try:
-                os.remove(requirement)
-            except OSError:
-                pass
+        for tempdir in [cr for cr in cleanup_requirements if cr is not None]:
+            if os.path.isdir(tempdir):
+                shutil.rmtree(tempdir)
 
 
 def uninstall(pkgs=None,
@@ -743,7 +838,6 @@ def uninstall(pkgs=None,
               user=None,
               no_chown=False,
               cwd=None,
-              __env__=None,
               saltenv='base',
               use_vt=False):
     '''
@@ -783,7 +877,7 @@ def uninstall(pkgs=None,
     cwd
         Current working directory to run pip from
     use_vt
-        Use VT terminal emulation (see ouptut while installing)
+        Use VT terminal emulation (see output while installing)
 
     CLI Example:
 
@@ -799,19 +893,11 @@ def uninstall(pkgs=None,
 
     cmd = [pip_bin, 'uninstall', '-y']
 
-    if isinstance(__env__, string_types):
-        salt.utils.warn_until(
-            'Boron',
-            'Passing a salt environment should be done using \'saltenv\' '
-            'not \'__env__\'. This functionality will be removed in Salt '
-            'Boron.'
-        )
-        # Backwards compatibility
-        saltenv = __env__
+    cleanup_requirements, error = _process_requirements(
+        requirements=requirements, cmd=cmd, saltenv=saltenv, user=user,
+        cwd=cwd
+    )
 
-    cleanup_requirements, error = _process_requirements(requirements=requirements, cmd=cmd,
-                                                        saltenv=saltenv, user=user,
-                                                        no_chown=no_chown)
     if error:
         return error
 
@@ -842,7 +928,7 @@ def uninstall(pkgs=None,
         cmd.extend(['--timeout', timeout])
 
     if pkgs:
-        if isinstance(pkgs, string_types):
+        if isinstance(pkgs, six.string_types):
             pkgs = [p.strip() for p in pkgs.split(',')]
         if requirements:
             for requirement in requirements:
@@ -856,7 +942,8 @@ def uninstall(pkgs=None,
                             pass
         cmd.extend(pkgs)
 
-    cmd_kwargs = dict(python_shell=False, runas=user, cwd=cwd, saltenv=saltenv, use_vt=use_vt)
+    cmd_kwargs = dict(python_shell=False, runas=user,
+                      cwd=cwd, saltenv=saltenv, use_vt=use_vt)
     if bin_env and os.path.isdir(bin_env):
         cmd_kwargs['env'] = {'VIRTUAL_ENV': bin_env}
 
@@ -864,10 +951,11 @@ def uninstall(pkgs=None,
         return __salt__['cmd.run_all'](cmd, **cmd_kwargs)
     finally:
         for requirement in cleanup_requirements:
-            try:
-                os.remove(requirement)
-            except OSError:
-                pass
+            if requirement:
+                try:
+                    os.remove(requirement)
+                except OSError:
+                    pass
 
 
 def freeze(bin_env=None,
@@ -889,15 +977,40 @@ def freeze(bin_env=None,
     cwd
         Current working directory to run pip from
 
+    .. note::
+
+        If the version of pip available is older than 8.0.3, the list will not
+        include the packages pip, wheel, setuptools, or distribute even if they
+        are installed.
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' pip.freeze /home/code/path/to/virtualenv/
+
+    .. versionchanged:: 2016.11.2
+
+        The packages pip, wheel, setuptools, and distribute are included if the
+        installed pip is new enough.
     '''
     pip_bin = _get_pip_bin(bin_env)
 
     cmd = [pip_bin, 'freeze']
+
+    # Include pip, setuptools, distribute, wheel
+    min_version = '8.0.3'
+    cur_version = version(bin_env)
+    if not salt.utils.compare_versions(ver1=cur_version, oper='>=',
+                                       ver2=min_version):
+        logger.warning(
+            ('The version of pip installed is {0}, which is older than {1}. '
+             'The packages pip, wheel, setuptools, and distribute will not be '
+             'included in the output of pip.freeze').format(cur_version,
+                                                            min_version))
+    else:
+        cmd.append('--all')
+
     cmd_kwargs = dict(runas=user, cwd=cwd, use_vt=use_vt, python_shell=False)
     if bin_env and os.path.isdir(bin_env):
         cmd_kwargs['env'] = {'VIRTUAL_ENV': bin_env}
@@ -917,30 +1030,31 @@ def list_(prefix=None,
     Filter list of installed apps from ``freeze`` and check to see if
     ``prefix`` exists in the list of packages installed.
 
+    .. note::
+
+        If the version of pip available is older than 8.0.3, the packages
+        wheel, setuptools, and distribute will not be reported by this function
+        even if they are installed. Unlike
+        :py:func:`pip.freeze <salt.modules.pip.freeze>`, this function always
+        reports the version of pip which is installed.
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' pip.list salt
+
+    .. versionchanged:: 2016.11.2
+
+        The packages wheel, setuptools, and distribute are included if the
+        installed pip is new enough.
     '''
     packages = {}
 
-    pip_bin = _get_pip_bin(bin_env)
-
-    cmd = [pip_bin, 'freeze']
-
-    cmd_kwargs = dict(runas=user, cwd=cwd, python_shell=False)
-    if bin_env and os.path.isdir(bin_env):
-        cmd_kwargs['env'] = {'VIRTUAL_ENV': bin_env}
-
-    if not prefix or prefix in ('p', 'pi', 'pip'):
+    if prefix is None or 'pip'.startswith(prefix):
         packages['pip'] = version(bin_env)
 
-    result = __salt__['cmd.run_all'](cmd, **cmd_kwargs)
-    if result['retcode'] > 0:
-        raise CommandExecutionError(result['stderr'])
-
-    for line in result['stdout'].splitlines():
+    for line in freeze(bin_env=bin_env, user=user, cwd=cwd):
         if line.startswith('-f') or line.startswith('#'):
             # ignore -f line as it contains --find-links directory
             # ignore comment lines
@@ -951,6 +1065,9 @@ def list_(prefix=None,
         elif line.startswith('-e'):
             line = line.split('-e ')[1]
             version_, name = line.split('#egg=')
+        elif len(line.split('===')) >= 2:
+            name = line.split('===')[0]
+            version_ = line.split('===')[1]
         elif len(line.split('==')) >= 2:
             name = line.split('==')[0]
             version_ = line.split('==')[1]
@@ -963,6 +1080,7 @@ def list_(prefix=None,
                 packages[name] = version_
         else:
             packages[name] = version_
+
     return packages
 
 
@@ -983,7 +1101,8 @@ def version(bin_env=None):
     '''
     pip_bin = _get_pip_bin(bin_env)
 
-    output = __salt__['cmd.run']('{0} --version'.format(pip_bin), python_shell=False)
+    output = __salt__['cmd.run'](
+        '{0} --version'.format(pip_bin), python_shell=False)
     try:
         return re.match(r'^pip (\S+)', output).group(1)
     except AttributeError:
