@@ -129,8 +129,6 @@ def create(path,
     '''
     if venv_bin is None:
         venv_bin = __opts__.get('venv_bin') or __pillar__.get('venv_bin')
-    # raise CommandNotFoundError if venv_bin is missing
-    salt.utils.check_or_die(venv_bin)
 
     cmd = [venv_bin]
 
@@ -382,7 +380,7 @@ def get_resource_path(venv,
     package_or_requirement
         Name of the package in which the resource resides
 
-        .. deprecated:: Nitrogen
+        .. deprecated:: 2016.3.0
             Use ``package`` instead.
 
     resource
@@ -393,7 +391,7 @@ def get_resource_path(venv,
     resource_name
         Name of the resource of which the path is to be returned
 
-        .. deprecated:: Nitrogen
+        .. deprecated:: 2016.3.0
 
 
     .. versionadded:: 2015.5.0
@@ -473,7 +471,7 @@ def get_resource_content(venv,
     package_or_requirement
         Name of the package in which the resource resides
 
-        .. deprecated:: Nitrogen
+        .. deprecated:: 2016.3.0
             Use ``package`` instead.
 
     resource
@@ -484,7 +482,7 @@ def get_resource_content(venv,
     resource_name
         Name of the resource of which the content is to be returned
 
-        .. deprecated:: Nitrogen
+        .. deprecated:: 2016.3.0
 
 
     .. versionadded:: 2015.5.0
