@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 '''
     :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: :email:`Alexandru Bleotu (alexandru.bleotu@morganstanley.com)`
 
 
     salt.utils.schema
@@ -8,14 +9,23 @@
 
     Object Oriented Configuration - JSON Schema compatible generator
 
-    This code was inspired by `jsl`__, "A Python DSL for describing JSON schemas".
+    This code was inspired by `jsl`__, "A Python DSL for describing JSON
+    schemas".
 
-    .. __: http://jsl.readthedocs.org/
+    .. __: https://jsl.readthedocs.io/
 
 
-    A configuration document or configuration document section is defined using the
-    py:class:`Schema`, the configuration items are defined by any of the subclasses
-    of py:class:`BaseSchemaItem` as attributes of a subclass of py:class:`Schema` class.
+    A configuration document or configuration document section is defined using
+    the py:class:`Schema`, the configuration items are defined by any of the
+    subclasses of py:class:`BaseSchemaItem` as attributes of a subclass of
+    py:class:`Schema` class.
+
+    A more complex configuration document (containing a defininitions section)
+    is defined using the py:class:`DefinitionsSchema`. This type of
+    schema supports having complex configuration items as attributes (defined
+    extending the py:class:`ComplexSchemaItem`). These items have other
+    configuration items (complex or not) as attributes, allowing to verify
+    more complex JSON data structures
 
     As an example:
 
@@ -100,8 +110,8 @@
         }
 
 
-    The serialized version of the configuration block can be used to validate a configuration dictionary using
-    the `python jsonschema library`__.
+    The serialized version of the configuration block can be used to validate a
+    configuration dictionary using the `python jsonschema library`__.
 
     .. __: https://pypi.python.org/pypi/jsonschema
 
@@ -252,9 +262,10 @@
             -1
         >>>
 
-    If however, you just want to use the configuration blocks for readability and do not desire the nested
-    dictionaries serialization, you can pass ``flatten=True`` when defining a configuration section as a
-    configuration subclass attribute:
+    If however, you just want to use the configuration blocks for readability
+    and do not desire the nested dictionaries serialization, you can pass
+    ``flatten=True`` when defining a configuration section as a configuration
+    subclass attribute:
 
     .. code-block:: python
 
@@ -316,6 +327,7 @@ import textwrap
 import functools
 
 # Import salt libs
+import salt.utils.args
 from salt.utils.odict import OrderedDict
 
 # Import 3rd-party libs
@@ -405,6 +417,7 @@ class SchemaMeta(six.with_metaclass(Prepareable, type)):
         # Mark the instance as a configuration document/section
         attrs['__config__'] = True
         attrs['__flatten__'] = False
+        attrs['__config_name__'] = None
 
         # Let's record the configuration items/sections
         items = {}
@@ -421,6 +434,7 @@ class SchemaMeta(six.with_metaclass(Prepareable, type)):
 
         # Iterate through attrs to discover items/config sections
         for key, value in six.iteritems(attrs):
+            entry_name = None
             if not hasattr(value, '__item__') and not hasattr(value, '__config__'):
                 continue
             if hasattr(value, '__item__'):
@@ -429,10 +443,12 @@ class SchemaMeta(six.with_metaclass(Prepareable, type)):
                     # It's an item instance without a title, make the title
                     # it's name
                     value.title = key
-                items[key] = value
+                entry_name = value.__item_name__ or key
+                items[entry_name] = value
             if hasattr(value, '__config__'):
-                sections[key] = value
-            order.append(key)
+                entry_name = value.__config_name__ or key
+                sections[entry_name] = value
+            order.append(entry_name)
 
         attrs['_order'] = order
         attrs['_items'] = items
@@ -441,6 +457,7 @@ class SchemaMeta(six.with_metaclass(Prepareable, type)):
 
     def __call__(cls, flatten=False, allow_additional_items=False, **kwargs):
         instance = object.__new__(cls)
+        instance.__config_name__ = kwargs.pop('name', None)
         if flatten is True:
             # This configuration block is to be treated as a part of the
             # configuration for which it was defined as an attribute, not as
@@ -466,15 +483,21 @@ class BaseSchemaItemMeta(six.with_metaclass(Prepareable, type)):
     def __new__(mcs, name, bases, attrs):
         # Register the class as an item class
         attrs['__item__'] = True
+        attrs['__item_name__'] = None
         # Instantiate an empty list to store the config item attribute names
         attributes = []
         for base in reversed(bases):
             try:
+                base_attributes = getattr(base, '_attributes', [])
+                if base_attributes:
+                    attributes.extend(base_attributes)
                 # Extend the attributes with the base argspec argument names
                 # but skip "self"
-                for argname in inspect.getargspec(base.__init__).args:
+                for argname in salt.utils.args.get_function_argspec(base.__init__).args:
                     if argname == 'self' or argname in attributes:
                         continue
+                    if argname == 'name':
+                        continue
                     attributes.append(argname)
             except TypeError:
                 # On the base object type, __init__ is just a wrapper which
@@ -492,9 +515,13 @@ class BaseSchemaItemMeta(six.with_metaclass(Prepareable, type)):
                 'Please pass all arguments as named arguments. Un-named '
                 'arguments are not supported'
             )
-        for key in kwargs.keys():
+        for key in kwargs.copy().keys():
             # Store the kwarg keys as the instance attributes for the
             # serialization step
+            if key == 'name':
+                # This is the item name to override the class attribute name
+                instance.__item_name__ = kwargs.pop(key)
+                continue
             if key not in instance._attributes:
                 instance._attributes.append(key)
         # Init the class
@@ -520,7 +547,7 @@ class Schema(six.with_metaclass(SchemaMeta, object)):
     # Define some class level attributes to make PyLint happier
     title = None
     description = None
-    _items = _sections = None
+    _items = _sections = _order = None
     __flatten__ = False
     __allow_additional_items__ = False
 
@@ -547,8 +574,9 @@ class Schema(six.with_metaclass(SchemaMeta, object)):
         serialized['type'] = 'object'
         properties = OrderedDict()
         cls.after_items_update = []
-        for name in cls._order:
+        for name in cls._order:  # pylint: disable=E1133
             skip_order = False
+            item_name = None
             if name in cls._sections:  # pylint: disable=E1135
                 section = cls._sections[name]
                 serialized_section = section.serialize(None if section.__flatten__ is True else name)
@@ -569,22 +597,27 @@ class Schema(six.with_metaclass(SchemaMeta, object)):
 
             if name in cls._items:  # pylint: disable=E1135
                 config = cls._items[name]
+                item_name = config.__item_name__ or name
                 # Handle the configuration items defined in the class instance
                 if config.__flatten__ is True:
                     serialized_config = config.serialize()
                     cls.after_items_update.append(serialized_config)
                     skip_order = True
                 else:
-                    properties[name] = config.serialize()
+                    properties[item_name] = config.serialize()
 
                 if config.required:
                     # If it's a required item, add it to the required list
-                    required.append(name)
+                    required.append(item_name)
 
             if skip_order is False:
                 # Store the order of the item
-                if name not in ordering:
-                    ordering.append(name)
+                if item_name is not None:
+                    if item_name not in ordering:
+                        ordering.append(item_name)
+                else:
+                    if name not in ordering:
+                        ordering.append(name)
 
         if properties:
             serialized['properties'] = properties
@@ -844,6 +877,11 @@ class BaseSchemaItem(SchemaItem):
     #    return output + '\n'
 
 
+class NullItem(BaseSchemaItem):
+
+    __type__ = 'null'
+
+
 class BooleanItem(BaseSchemaItem):
     __type__ = 'boolean'
 
@@ -1009,11 +1047,11 @@ class NumberItem(BaseSchemaItem):
         :param minimum:
             The minimum allowed value
         :param exclusive_minimum:
-            Wether a value is allowed to be exactly equal to the minimum
+            Whether a value is allowed to be exactly equal to the minimum
         :param maximum:
             The maximum allowed value
         :param exclusive_maximum:
-            Wether a value is allowed to be exactly equal to the maximum
+            Whether a value is allowed to be exactly equal to the maximum
         '''
         if multiple_of is not None:
             self.multiple_of = multiple_of
@@ -1097,24 +1135,25 @@ class ArrayItem(BaseSchemaItem):
         super(ArrayItem, self).__init__(**kwargs)
 
     def __validate_attributes__(self):
-        if not self.items:
+        if not self.items and not self.additional_items:
             raise RuntimeError(
-                'The passed items must not be empty'
-            )
-        if isinstance(self.items, (list, tuple)):
-            for item in self.items:
-                if not isinstance(item, (Schema, SchemaItem)):
-                    raise RuntimeError(
-                        'All items passed in the item argument tuple/list must be '
-                        'a subclass of Schema, SchemaItem or BaseSchemaItem, '
-                        'not {0}'.format(type(item))
-                    )
-        elif not isinstance(self.items, (Schema, SchemaItem)):
-            raise RuntimeError(
-                'The items argument passed must be a subclass of '
-                'Schema, SchemaItem or BaseSchemaItem, not '
-                '{0}'.format(type(self.items))
+                'One of items or additional_items must be passed.'
             )
+        if self.items is not None:
+            if isinstance(self.items, (list, tuple)):
+                for item in self.items:
+                    if not isinstance(item, (Schema, SchemaItem)):
+                        raise RuntimeError(
+                            'All items passed in the item argument tuple/list must be '
+                            'a subclass of Schema, SchemaItem or BaseSchemaItem, '
+                            'not {0}'.format(type(item))
+                        )
+            elif not isinstance(self.items, (Schema, SchemaItem)):
+                raise RuntimeError(
+                    'The items argument passed must be a subclass of '
+                    'Schema, SchemaItem or BaseSchemaItem, not '
+                    '{0}'.format(type(self.items))
+                )
 
     def __get_items__(self):
         if isinstance(self.items, (Schema, SchemaItem)):
@@ -1200,9 +1239,9 @@ class DictItem(BaseSchemaItem):
         super(DictItem, self).__init__(**kwargs)
 
     def __validate_attributes__(self):
-        if not self.properties and not self.pattern_properties:
+        if not self.properties and not self.pattern_properties and not self.additional_properties:
             raise RuntimeError(
-                'One of properties or pattern properties must be passed'
+                'One of properties, pattern_properties or additional_properties must be passed'
             )
         if self.properties is not None:
             if not isinstance(self.properties, (Schema, dict)):
@@ -1411,3 +1450,114 @@ class PortItem(IntegerItem):
     minimum = 0  # yes, 0 is a valid port number
     maximum = 65535
 # <---- Custom Preconfigured Configs ---------------------------------------------------------------------------------
+
+
+class ComplexSchemaItem(BaseSchemaItem):
+    '''
+    .. versionadded:: 2016.11.0
+
+    Complex Schema Item
+
+    This item can contain other schema items as attributes; the names of all
+    attributes need to be included in the _``_complex_attributes`` attribute
+    '''
+
+    # This attribute is populated by the metaclass, but pylint fails to see it
+    # and assumes it's not an iterable
+    _attributes = []
+    _complex_attributes = []
+    _definition_name = None
+
+    def __init__(self, definition_name=None):
+        super(ComplexSchemaItem, self).__init__()
+        self.__type__ = 'object'
+        self._definition_name = definition_name if definition_name else \
+                self.__class__.__name__
+        if self._complex_attributes:
+            for complex_attr in self._complex_attributes:
+                if complex_attr not in self._attributes:
+                    self._attributes.append(complex_attr)
+
+    @property
+    def definition_name(self):
+        '''
+        Definition name property
+        '''
+        return self._definition_name
+
+    def serialize(self):
+        '''
+        The serialization of the complex item is a pointer to the item
+        definition
+        '''
+        return {'$ref': '#/definitions/{0}'.format(self.definition_name)}
+
+    def get_definition(self):
+        '''Returns the definition of the complex item'''
+
+        serialized = super(ComplexSchemaItem, self).serialize()
+        # Adjust entries in the serialization
+        del serialized['definition_name']
+        serialized['title'] = self.definition_name
+
+        properties = {}
+        required_attr_names = []
+
+        for attr_name in self._attributes:
+            attr = getattr(self, attr_name)
+            if attr and isinstance(attr, BaseSchemaItem):
+                # Remove the attribute entry added by the base serialization
+                del serialized[attr_name]
+                properties[attr_name] = attr.serialize()
+                properties[attr_name]['type'] = attr.__type__
+                if attr.required:
+                    required_attr_names.append(attr_name)
+        if serialized.get('properties') is None:
+            serialized['properties'] = {}
+        serialized['properties'].update(properties)
+
+        # Assign the required array
+        if required_attr_names:
+            serialized['required'] = required_attr_names
+        return serialized
+
+    def get_complex_attrs(self):
+        '''Returns a dictionary of the complex attributes'''
+        return [getattr(self, attr_name) for attr_name in self._attributes if
+                isinstance(getattr(self, attr_name), ComplexSchemaItem)]
+
+
+class DefinitionsSchema(Schema):
+    '''
+    .. versionadded:: 2016.11.0
+
+    JSON schema classs that supports ComplexSchemaItem objects by adding
+    a definitions section to the JSON schema, containing the item definitions.
+
+    All references to ComplexSchemaItems are built using schema inline
+    dereferencing.
+    '''
+
+    @classmethod
+    def serialize(cls, id_=None):
+        # Get the initial serialization
+        serialized = super(DefinitionsSchema, cls).serialize(id_)
+
+        # Augment the serializations with the definitions of all complex items
+        complex_items = [config for config in six.itervalues(cls._items)
+                         if isinstance(config, ComplexSchemaItem)]
+        aux_items = complex_items[:]
+        while aux_items:
+            item = aux_items.pop(0)
+            # Recursively add complex attributes
+            new_items = item.get_complex_attrs()
+            complex_items.extend(new_items)
+            aux_items.extend(new_items)
+
+        definitions = OrderedDict()
+        for config in complex_items:
+            if isinstance(config, ComplexSchemaItem):
+                definitions[config.definition_name] = \
+                        config.get_definition()
+        serialized['definitions'] = definitions
+        return serialized
