@@ -23,15 +23,15 @@ class ManageTest(integration.ShellCase):
         jobs.active
         '''
         ret = self.run_run_plus('jobs.active')
-        self.assertEqual(ret['fun'], {})
+        self.assertEqual(ret['return'], {})
         self.assertEqual(ret['out'], [])
 
     def test_lookup_jid(self):
         '''
         jobs.lookup_jid
         '''
-        ret = self.run_run_plus('jobs.lookup_jid', '', '23974239742394')
-        self.assertEqual(ret['fun'], {})
+        ret = self.run_run_plus('jobs.lookup_jid', '23974239742394')
+        self.assertEqual(ret['return'], {})
         self.assertEqual(ret['out'], [])
 
     @skipIf(True, 'to be reenabled when #23623 is merged')
@@ -40,7 +40,7 @@ class ManageTest(integration.ShellCase):
         jobs.list_jobs
         '''
         ret = self.run_run_plus('jobs.list_jobs')
-        self.assertIsInstance(ret['fun'], dict)
+        self.assertIsInstance(ret['return'], dict)
 
 
 if __name__ == '__main__':
