@@ -22,34 +22,70 @@ from salt.ext.six.moves import range
 ensure_in_syspath('../../')
 
 # Import Salt Libs
-from salt.modules import dockerng as dockerng_mod
+import salt.modules.dockerng as dockerng_mod
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 
-dockerng_mod.__context__ = {'docker.docker_version': ''}
 dockerng_mod.__salt__ = {}
 dockerng_mod.__opts__ = {}
 
 
-def _docker_py_version():
-    return dockerng_mod.docker.version_info if dockerng_mod.HAS_DOCKER_PY else None
-
-
 @skipIf(NO_MOCK, NO_MOCK_REASON)
+@skipIf(dockerng_mod.HAS_DOCKER_PY is False, 'docker-py must be installed to run these tests. Skipping.')
 class DockerngTestCase(TestCase):
     '''
     Validate dockerng module
     '''
+    def setUp(self):
+        '''
+        Ensure we aren't persisting context dunders between tests
+        '''
+        dockerng_mod.__context__ = {'docker.docker_version': ''}
+
+    try:
+        docker_version = dockerng_mod.docker.version_info
+    except AttributeError:
+        docker_version = 0,
+
+    client_args_mock = MagicMock(return_value={
+        'create_container': [
+            'image', 'command', 'hostname', 'user', 'detach', 'stdin_open',
+            'tty', 'ports', 'environment', 'volumes', 'network_disabled',
+            'name', 'entrypoint', 'working_dir', 'domainname', 'cpuset',
+            'host_config', 'mac_address', 'labels', 'volume_driver',
+            'stop_signal', 'networking_config', 'healthcheck',
+            'stop_timeout'],
+        'host_config': [
+            'binds', 'port_bindings', 'lxc_conf', 'publish_all_ports',
+            'links', 'privileged', 'dns', 'dns_search', 'volumes_from',
+            'network_mode', 'restart_policy', 'cap_add', 'cap_drop',
+            'devices', 'extra_hosts', 'read_only', 'pid_mode', 'ipc_mode',
+            'security_opt', 'ulimits', 'log_config', 'mem_limit',
+            'memswap_limit', 'mem_reservation', 'kernel_memory',
+            'mem_swappiness', 'cgroup_parent', 'group_add', 'cpu_quota',
+            'cpu_period', 'blkio_weight', 'blkio_weight_device',
+            'device_read_bps', 'device_write_bps', 'device_read_iops',
+            'device_write_iops', 'oom_kill_disable', 'shm_size', 'sysctls',
+            'tmpfs', 'oom_score_adj', 'dns_opt', 'cpu_shares',
+            'cpuset_cpus', 'userns_mode', 'pids_limit', 'isolation',
+            'auto_remove', 'storage_opt'],
+        'networking_config': [
+            'aliases', 'links', 'ipv4_address', 'ipv6_address',
+            'link_local_ips'],
+    })
 
     def test_ps_with_host_true(self):
         '''
         Check that dockerng.ps called with host is ``True``,
         include resutlt of ``network.interfaces`` command in returned result.
         '''
+        client = Mock()
+        client.containers = MagicMock(return_value=[])
+        get_client_mock = MagicMock(return_value=client)
+
         network_interfaces = Mock(return_value={'mocked': None})
         with patch.dict(dockerng_mod.__salt__,
                         {'network.interfaces': network_interfaces}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': MagicMock()}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 ret = dockerng_mod.ps_(host=True)
                 self.assertEqual(ret,
                                  {'host': {'interfaces': {'mocked': None}}})
@@ -58,15 +94,16 @@ class DockerngTestCase(TestCase):
         '''
         Check that dockerng.ps accept filters parameter.
         '''
-        client = MagicMock()
-        with patch.dict(dockerng_mod.__context__,
-                        {'docker.client': client}):
+        client = Mock()
+        client.containers = MagicMock(return_value=[])
+        get_client_mock = MagicMock(return_value=client)
+
+        with patch.object(dockerng_mod, '_get_client', get_client_mock):
             dockerng_mod.ps_(filters={'label': 'KEY'})
             client.containers.assert_called_once_with(
                 all=True,
                 filters={'label': 'KEY'})
 
-    @skipIf(_docker_py_version() is None, 'docker-py needs to be installed for this test to run')
     @patch.object(dockerng_mod, '_get_exec_driver')
     def test_check_mine_cache_is_refreshed_on_container_change_event(self, _):
         '''
@@ -87,19 +124,21 @@ class DockerngTestCase(TestCase):
                                    ):
             mine_send = Mock()
             command = getattr(dockerng_mod, command_name)
-            docker_client = MagicMock()
-            docker_client.api_version = '1.12'
+            client = MagicMock()
+            client.api_version = '1.12'
+            get_client_mock = MagicMock(return_value=client)
+
             with patch.dict(dockerng_mod.__salt__,
                             {'mine.send': mine_send,
                              'container_resource.run': MagicMock(),
                              'cp.cache_file': MagicMock(return_value=False)}):
-                with patch.dict(dockerng_mod.__context__,
-                                {'docker.client': docker_client}):
-                    command('container', *args)
+                with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                    with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                        command('container', *args)
             mine_send.assert_called_with('dockerng.ps', verbose=True, all=True,
                                          host=True)
 
-    @skipIf(_docker_py_version() < (1, 4, 0),
+    @skipIf(docker_version < (1, 4, 0),
             'docker module must be installed to run this test or is too old. >=1.4.0')
     @patch.object(dockerng_mod, 'images', MagicMock())
     @patch.object(dockerng_mod, 'inspect_image')
@@ -117,18 +156,19 @@ class DockerngTestCase(TestCase):
         client.api_version = '1.19'
         client.create_host_config.return_value = host_config
         client.create_container.return_value = {}
-        with patch.dict(dockerng_mod.__dict__,
-                        {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.create('image', cmd='ls', name='ctn')
+        get_client_mock = MagicMock(return_value=client)
+
+        with patch.dict(dockerng_mod.__dict__, {'__salt__': __salt__}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                    dockerng_mod.create('image', cmd='ls', name='ctn')
         client.create_container.assert_called_once_with(
             command='ls',
             host_config=host_config,
             image='image',
             name='ctn')
 
-    @skipIf(_docker_py_version() < (1, 4, 0),
+    @skipIf(docker_version < (1, 4, 0),
             'docker module must be installed to run this test or is too old. >=1.4.0')
     @patch.object(dockerng_mod, 'images', MagicMock())
     @patch.object(dockerng_mod, 'inspect_image')
@@ -146,17 +186,18 @@ class DockerngTestCase(TestCase):
         client.api_version = '1.19'
         client.create_host_config.return_value = host_config
         client.create_container.return_value = {}
-        with patch.dict(dockerng_mod.__dict__,
-                        {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.create('image', name='ctn', publish_all_ports=True)
+        get_client_mock = MagicMock(return_value=client)
+
+        with patch.dict(dockerng_mod.__dict__, {'__salt__': __salt__}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                    dockerng_mod.create('image', name='ctn', publish_all_ports=True)
         client.create_container.assert_called_once_with(
             host_config=host_config,
             image='image',
             name='ctn')
 
-    @skipIf(_docker_py_version() < (1, 4, 0),
+    @skipIf(docker_version < (1, 4, 0),
             'docker module must be installed to run this test or is too old. >=1.4.0')
     @patch.object(dockerng_mod, 'images', MagicMock())
     @patch.object(dockerng_mod, 'inspect_image')
@@ -168,22 +209,24 @@ class DockerngTestCase(TestCase):
         __salt__ = {
             'config.get': Mock(),
             'mine.send': Mock(),
+            'dockerng.version': MagicMock(return_value={}),
         }
         host_config = {}
         client = Mock()
         client.api_version = '1.19'
         client.create_host_config.return_value = host_config
         client.create_container.return_value = {}
-        with patch.dict(dockerng_mod.__dict__,
-                        {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.create(
-                    'image',
-                    name='ctn',
-                    labels={'KEY': 'VALUE'},
-                    validate_input=True,
-                )
+        get_client_mock = MagicMock(return_value=client)
+
+        with patch.dict(dockerng_mod.__dict__, {'__salt__': __salt__}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                    dockerng_mod.create(
+                        'image',
+                        name='ctn',
+                        labels={'KEY': 'VALUE'},
+                        validate_input=True,
+                    )
         client.create_container.assert_called_once_with(
             labels={'KEY': 'VALUE'},
             host_config=host_config,
@@ -191,7 +234,7 @@ class DockerngTestCase(TestCase):
             name='ctn',
         )
 
-    @skipIf(_docker_py_version() < (1, 4, 0),
+    @skipIf(docker_version < (1, 4, 0),
             'docker module must be installed to run this test or is too old. >=1.4.0')
     @patch.object(dockerng_mod, 'images', MagicMock())
     @patch.object(dockerng_mod, 'inspect_image')
@@ -203,22 +246,23 @@ class DockerngTestCase(TestCase):
         __salt__ = {
             'config.get': Mock(),
             'mine.send': Mock(),
+            'dockerng.version': MagicMock(return_value={}),
         }
         host_config = {}
         client = Mock()
         client.api_version = '1.19'
         client.create_host_config.return_value = host_config
         client.create_container.return_value = {}
-        with patch.dict(dockerng_mod.__dict__,
-                        {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.create(
-                    'image',
-                    name='ctn',
-                    labels=['KEY1', 'KEY2'],
-                    validate_input=True,
-                )
+        get_client_mock = MagicMock(return_value=client)
+        with patch.dict(dockerng_mod.__dict__, {'__salt__': __salt__}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                    dockerng_mod.create(
+                        'image',
+                        name='ctn',
+                        labels=['KEY1', 'KEY2'],
+                        validate_input=True,
+                    )
         client.create_container.assert_called_once_with(
             labels=['KEY1', 'KEY2'],
             host_config=host_config,
@@ -226,7 +270,7 @@ class DockerngTestCase(TestCase):
             name='ctn',
         )
 
-    @skipIf(_docker_py_version() < (1, 4, 0),
+    @skipIf(docker_version < (1, 4, 0),
             'docker module must be installed to run this test or is too old. >=1.4.0')
     @patch.object(dockerng_mod, 'images', MagicMock())
     @patch.object(dockerng_mod, 'inspect_image')
@@ -238,25 +282,28 @@ class DockerngTestCase(TestCase):
         __salt__ = {
             'config.get': Mock(),
             'mine.send': Mock(),
+            'dockerng.version': MagicMock(return_value={}),
         }
         host_config = {}
         client = Mock()
         client.api_version = '1.19'
         client.create_host_config.return_value = host_config
         client.create_container.return_value = {}
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                self.assertRaises(SaltInvocationError,
-                                  dockerng_mod.create,
-                                  'image',
-                                  name='ctn',
-                                  labels=22,
-                                  validate_input=True,
-                                  )
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                    self.assertRaises(SaltInvocationError,
+                                      dockerng_mod.create,
+                                      'image',
+                                      name='ctn',
+                                      labels=22,
+                                      validate_input=True,
+                                      )
 
-    @skipIf(_docker_py_version() < (1, 4, 0),
+    @skipIf(docker_version < (1, 4, 0),
             'docker module must be installed to run this test or is too old. >=1.4.0')
     @patch.object(dockerng_mod, 'images', MagicMock())
     @patch.object(dockerng_mod, 'inspect_image')
@@ -268,22 +315,24 @@ class DockerngTestCase(TestCase):
         __salt__ = {
             'config.get': Mock(),
             'mine.send': Mock(),
+            'dockerng.version': MagicMock(return_value={}),
         }
         host_config = {}
         client = Mock()
         client.api_version = '1.19'
         client.create_host_config.return_value = host_config
         client.create_container.return_value = {}
-        with patch.dict(dockerng_mod.__dict__,
-                        {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.create(
-                    'image',
-                    name='ctn',
-                    labels=[{'KEY1': 'VALUE1'}, {'KEY2': 'VALUE2'}],
-                    validate_input=True,
-                )
+        get_client_mock = MagicMock(return_value=client)
+
+        with patch.dict(dockerng_mod.__dict__, {'__salt__': __salt__}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                with patch.object(dockerng_mod, 'get_client_args', self.client_args_mock):
+                    dockerng_mod.create(
+                        'image',
+                        name='ctn',
+                        labels=[{'KEY1': 'VALUE1'}, {'KEY2': 'VALUE2'}],
+                        validate_input=True,
+                    )
         client.create_container.assert_called_once_with(
             labels={'KEY1': 'VALUE1', 'KEY2': 'VALUE2'},
             host_config=host_config,
@@ -291,8 +340,10 @@ class DockerngTestCase(TestCase):
             name='ctn',
         )
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_list_networks(self, *args):
         '''
         test list networks.
@@ -304,10 +355,11 @@ class DockerngTestCase(TestCase):
         host_config = {}
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.networks(
                     names=['foo'],
                     ids=['01234'],
@@ -317,8 +369,10 @@ class DockerngTestCase(TestCase):
                     ids=['01234'],
         )
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_create_network(self, *args):
         '''
         test create network.
@@ -330,21 +384,18 @@ class DockerngTestCase(TestCase):
         host_config = {}
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.create_network(
-                    'foo',
-                    driver='bridge',
-                )
-        client.create_network.assert_called_once_with(
-                    'foo',
-                    driver='bridge',
-        )
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                dockerng_mod.create_network('foo', driver='bridge')
+        client.create_network.assert_called_once_with('foo', driver='bridge')
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_remove_network(self, *args):
         '''
         test remove network.
@@ -356,15 +407,18 @@ class DockerngTestCase(TestCase):
         host_config = {}
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.remove_network('foo')
         client.remove_network.assert_called_once_with('foo')
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_inspect_network(self, *args):
         '''
         test inspect network.
@@ -376,15 +430,18 @@ class DockerngTestCase(TestCase):
         host_config = {}
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.inspect_network('foo')
         client.inspect_network.assert_called_once_with('foo')
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_connect_container_to_network(self, *args):
         '''
         test inspect network.
@@ -396,16 +453,19 @@ class DockerngTestCase(TestCase):
         host_config = {}
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.connect_container_to_network('container', 'foo')
         client.connect_container_to_network.assert_called_once_with(
             'container', 'foo')
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_disconnect_container_from_network(self, *args):
         '''
         test inspect network.
@@ -417,16 +477,19 @@ class DockerngTestCase(TestCase):
         host_config = {}
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.disconnect_container_from_network('container', 'foo')
         client.disconnect_container_from_network.assert_called_once_with(
             'container', 'foo')
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_list_volumes(self, *args):
         '''
         test list volumes.
@@ -437,19 +500,18 @@ class DockerngTestCase(TestCase):
         }
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
-                dockerng_mod.volumes(
-                    filters={'dangling': [True]},
-                )
-        client.volumes.assert_called_once_with(
-            filters={'dangling': [True]},
-        )
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
+                dockerng_mod.volumes(filters={'dangling': [True]})
+        client.volumes.assert_called_once_with(filters={'dangling': [True]})
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_create_volume(self, *args):
         '''
         test create volume.
@@ -460,10 +522,11 @@ class DockerngTestCase(TestCase):
         }
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.create_volume(
                     'foo',
                     driver='bridge',
@@ -475,8 +538,10 @@ class DockerngTestCase(TestCase):
                     driver_opts={},
         )
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_remove_volume(self, *args):
         '''
         test remove volume.
@@ -487,15 +552,18 @@ class DockerngTestCase(TestCase):
         }
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.remove_volume('foo')
         client.remove_volume.assert_called_once_with('foo')
 
-    @skipIf(_docker_py_version() < (1, 5, 0),
+    @skipIf(docker_version < (1, 5, 0),
             'docker module must be installed to run this test or is too old. >=1.5.0')
+    @patch('salt.modules.dockerng._get_docker_py_versioninfo',
+           MagicMock(return_value=docker_version))
     def test_inspect_volume(self, *args):
         '''
         test inspect volume.
@@ -506,10 +574,11 @@ class DockerngTestCase(TestCase):
         }
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
+
         with patch.dict(dockerng_mod.__dict__,
                         {'__salt__': __salt__}):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod.inspect_volume('foo')
         client.inspect_volume.assert_called_once_with('foo')
 
@@ -517,13 +586,14 @@ class DockerngTestCase(TestCase):
         client = Mock()
         client.api_version = '1.21'
         client.wait = Mock(return_value=0)
+        get_client_mock = MagicMock(return_value=client)
+
         dockerng_inspect_container = Mock(side_effect=[
             {'State': {'Running': True}},
             {'State': {'Stopped': True}}])
         with patch.object(dockerng_mod, 'inspect_container',
                           dockerng_inspect_container):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod._clear_context()
                 result = dockerng_mod.wait('foo')
         self.assertEqual(result, {'result': True,
@@ -535,14 +605,15 @@ class DockerngTestCase(TestCase):
         client = Mock()
         client.api_version = '1.21'
         client.wait = Mock(return_value=0)
+        get_client_mock = MagicMock(return_value=client)
+
         dockerng_inspect_container = Mock(side_effect=[
             {'State': {'Stopped': True}},
             {'State': {'Stopped': True}},
         ])
         with patch.object(dockerng_mod, 'inspect_container',
                           dockerng_inspect_container):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod._clear_context()
                 result = dockerng_mod.wait('foo')
         self.assertEqual(result, {'result': False,
@@ -555,14 +626,15 @@ class DockerngTestCase(TestCase):
         client = Mock()
         client.api_version = '1.21'
         client.wait = Mock(return_value=0)
+        get_client_mock = MagicMock(return_value=client)
+
         dockerng_inspect_container = Mock(side_effect=[
             {'State': {'Stopped': True}},
             {'State': {'Stopped': True}},
         ])
         with patch.object(dockerng_mod, 'inspect_container',
                           dockerng_inspect_container):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod._clear_context()
                 result = dockerng_mod.wait('foo', ignore_already_stopped=True)
         self.assertEqual(result, {'result': True,
@@ -574,11 +646,12 @@ class DockerngTestCase(TestCase):
     def test_wait_success_absent_container(self):
         client = Mock()
         client.api_version = '1.21'
+        get_client_mock = MagicMock(return_value=client)
         dockerng_inspect_container = Mock(side_effect=CommandExecutionError)
+
         with patch.object(dockerng_mod, 'inspect_container',
                           dockerng_inspect_container):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod._clear_context()
                 result = dockerng_mod.wait('foo', ignore_already_stopped=True)
         self.assertEqual(result, {'result': True,
@@ -588,13 +661,14 @@ class DockerngTestCase(TestCase):
         client = Mock()
         client.api_version = '1.21'
         client.wait = Mock(return_value=1)
+        get_client_mock = MagicMock(return_value=client)
         dockerng_inspect_container = Mock(side_effect=[
             {'State': {'Running': True}},
             {'State': {'Stopped': True}}])
+
         with patch.object(dockerng_mod, 'inspect_container',
                           dockerng_inspect_container):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod._clear_context()
                 result = dockerng_mod.wait('foo', fail_on_exit_status=True)
         self.assertEqual(result, {'result': False,
@@ -606,13 +680,14 @@ class DockerngTestCase(TestCase):
         client = Mock()
         client.api_version = '1.21'
         client.wait = Mock(return_value=1)
+        get_client_mock = MagicMock(return_value=client)
         dockerng_inspect_container = Mock(side_effect=[
             {'State': {'Stopped': True}},
             {'State': {'Stopped': True}}])
+
         with patch.object(dockerng_mod, 'inspect_container',
                           dockerng_inspect_container):
-            with patch.dict(dockerng_mod.__context__,
-                            {'docker.client': client}):
+            with patch.object(dockerng_mod, '_get_client', get_client_mock):
                 dockerng_mod._clear_context()
                 result = dockerng_mod.wait('foo',
                                            ignore_already_stopped=True,
@@ -752,8 +827,9 @@ class DockerngTestCase(TestCase):
                           {'Id': 'sha256:abcdef'},
                           {'Id': 'sha256:abcdefg',
                            'RepoTags': ['image:latest']}])
-        with patch.dict(dockerng_mod.__context__,
-                        {'docker.client': client}):
+        get_client_mock = MagicMock(return_value=client)
+
+        with patch.object(dockerng_mod, '_get_client', get_client_mock):
             dockerng_mod._clear_context()
             result = dockerng_mod.images()
         self.assertEqual(result,
