@@ -18,8 +18,7 @@ from salttesting.mock import (
 
 # Import Salt Libs
 from salt.modules import glusterfs
-import salt.utils.cloud as suc
-from salt.exceptions import CommandExecutionError, SaltInvocationError
+from salt.exceptions import SaltInvocationError
 
 # Globals
 glusterfs.__salt__ = {}
@@ -189,6 +188,7 @@ xml_peer_present = """
 <cliOutput>
   <opRet>0</opRet>
   <peer>
+    <uuid>uuid1</uuid>
     <hostname>node02</hostname>
     <hostnames>
       <hostname>node02.domain.dom</hostname>
@@ -311,6 +311,45 @@ xml_volume_info_stopped = """
 </cliOutput>
 """
 
+xml_peer_probe_success = """
+<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+<cliOutput>
+  <opRet>0</opRet>
+  <opErrno>0</opErrno>
+  <opErrstr/>
+  <output/>
+</cliOutput>
+"""
+
+xml_peer_probe_already_member = """
+<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+<cliOutput>
+  <opRet>0</opRet>
+  <opErrno>2</opErrno>
+  <opErrstr/>
+  <output>Host salt port 24007 already in peer list</output>
+</cliOutput>
+"""
+
+xml_peer_probe_localhost = """
+<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+<cliOutput>
+  <opRet>0</opRet>
+  <opErrno>1</opErrno>
+  <opErrstr/>
+  <output>Probe on localhost not needed</output>
+</cliOutput>
+"""
+
+xml_peer_probe_fail_cant_connect = """
+<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
+<cliOutput>
+  <opRet>-1</opRet>
+  <opErrno>107</opErrno>
+  <opErrstr>Probe returned with Transport endpoint is not connected</opErrstr>
+</cliOutput>
+"""
+
 xml_command_success = """
 <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
 <cliOutput>
@@ -334,208 +373,86 @@ class GlusterfsTestCase(TestCase):
     '''
     Test cases for salt.modules.glusterfs
     '''
-    # 'list_peers' function tests: 1
-
     maxDiff = None
 
-    def test_list_peers(self):
-        '''
-        Test if it return a list of gluster peers
-        '''
-        mock = MagicMock(return_value=xml_peer_present)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertDictEqual(glusterfs.list_peers(), {
-                                 'node02': ['node02.domain.dom', '10.0.0.2']})
+    # 'peer_status' function tests: 1
 
-        mock = MagicMock(return_value=xml_command_success)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertIsNone(glusterfs.list_peers())
+    def test_peer_status(self):
+        '''
+        Test gluster peer status
+        '''
+        mock_run = MagicMock(return_value=xml_peer_present)
+        with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+            self.assertDictEqual(
+                glusterfs.peer_status(),
+                {'uuid1': {
+                 'hostnames': ['node02', 'node02.domain.dom', '10.0.0.2']}})
+
+        mock_run = MagicMock(return_value=xml_command_success)
+        with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+            self.assertDictEqual(glusterfs.peer_status(), {})
 
     # 'peer' function tests: 1
 
     def test_peer(self):
         '''
-        Test if it adds another node into the peer list.
+        Test if gluster peer call is successful.
         '''
+        mock_run = MagicMock()
+        with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+            mock_run.return_value = xml_peer_probe_already_member
+            self.assertTrue(glusterfs.peer('salt'))
 
-        # invalid characters
-        mock = MagicMock(return_value=True)
-        with patch.object(suc, 'check_name', mock):
-            self.assertRaises(SaltInvocationError, glusterfs.peer, 'a')
-        # version 3.4
-        #   by hostname
-        #      peer failed unknown hostname
-        #      peer failed can't connect
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.fail_cant_connect)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(CommandExecutionError, glusterfs.peer, 'server2')
-        #      peer self
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_self)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server1'),
-                             {'exitval': '1', 'output': 'success: on localhost not needed'})
-        #      peer added
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_other)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server2'),
-                             {'exitval': '0', 'output': 'success'})
-        #      peer already member
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_already_peer['hostname'])
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server2'),
-                             {'exitval': '2', 'output': 'success: host server2 port 24007 already in peer list'})
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_already_peer['ip'])
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.2'),
-                             {'exitval': '2', 'output': 'success: host 10.0.0.2 port 24007 already in peer list'})
-        #      peer in reverse (probe server1 from server2)
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_first_hostname_from_second_first_time)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server1'),
-                             {'exitval': '0', 'output': 'success'})
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_first_hostname_from_second_second_time)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server1'),
-                             {'exitval': '2', 'output': 'success: host server1 port 24007 already in peer list'})
-        #      peer in reverse using ip address instead of hostname
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_reverse_already_peer['ip'])
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.1'),
-                             {'exitval': '2', 'output': 'success: host 10.0.0.1 port 24007 already in peer list'})
-        #   by ip address
-        #      peer self
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_self)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.1'),
-                             {'exitval': '1', 'output': 'success: on localhost not needed'})
-        #      peer added
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_other)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.2'),
-                             {'exitval': '0', 'output': 'success'})
-        #      peer already member
-        mock = MagicMock(
-            return_value=GlusterResults.v34.peer_probe.success_already_peer['ip'])
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.2'),
-                             {'exitval': '2', 'output': 'success: host 10.0.0.2 port 24007 already in peer list'})
-        # version 3.7
-        #      peer failed unknown hostname
-        #      peer failed can't connect
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.fail_cant_connect)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(CommandExecutionError, glusterfs.peer, 'server2')
-        #      peer self
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_self)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server1'),
-                             {'exitval': '1', 'output': 'Probe on localhost not needed'})
-        #      peer added
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_other)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server2'),
-                             {'exitval': '0', 'output': None})
-        #      peer already member
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_already_peer['hostname'])
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server2'),
-                             {'exitval': '2', 'output': 'Host server2 port 24007 already in peer list'})
-        #      peer in reverse
-        #   by ip address
-        #      peer added
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_other)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.2'),
-                             {'exitval': '0', 'output': None})
-        #      peer already member
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_already_peer['ip'])
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.2'),
-                             {'exitval': '2', 'output': 'Host 10.0.0.2 port 24007 already in peer list'})
-        #      peer self
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_self)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.1'),
-                             {'exitval': '1', 'output': 'Probe on localhost not needed'})
-        #      peer in reverse (probe server1 from server2)
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_first_hostname_from_second_first_time)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('server1'),
-                             {'exitval': '2', 'output': 'Host server1 port 24007 already in peer list'})
-        #      peer in reverse using ip address instead of hostname first time
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_first_ip_from_second_first_time)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.1'),
-                             {'exitval': '0', 'output': None})
-        mock = MagicMock(
-            return_value=GlusterResults.v37.peer_probe.success_first_ip_from_second_second_time)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.peer('10.0.0.1'),
-                             {'exitval': '2', 'output': 'Host 10.0.0.1 port 24007 already in peer list'})
+            mock_run.return_value = xml_peer_probe_localhost
+            self.assertTrue(glusterfs.peer('salt'))
 
-    # 'create' function tests: 1
+            mock_run.return_value = xml_peer_probe_fail_cant_connect
+            self.assertFalse(glusterfs.peer('salt'))
 
-    def test_create(self):
+    # 'create_volume' function tests: 1
+
+    def test_create_volume(self):
         '''
-        Test if it create a glusterfs volume.
+        Test if it creates a glusterfs volume.
         '''
-        mock = MagicMock(return_value='')
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
+        mock_run = MagicMock(return_value=xml_command_success)
+        with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
             self.assertRaises(
                 SaltInvocationError,
-                glusterfs.create,
+                glusterfs.create_volume,
                 'newvolume',
                 'host1:brick')
 
-        mock = MagicMock(return_value='')
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
             self.assertRaises(
                 SaltInvocationError,
-                glusterfs.create,
+                glusterfs.create_volume,
                 'newvolume',
                 'host1/brick')
 
-        mock = MagicMock(return_value=xml_command_fail)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                glusterfs.create,
-                'newvolume',
-                'host1:/brick',
-                True,
-                True,
-                True,
-                'tcp',
-                True)
+            self.assertFalse(mock_run.called)
 
-        mock = MagicMock(return_value=xml_command_success)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.create('newvolume', 'host1:/brick',
-                                              True, True, True, 'tcp', True),
-                             'Volume newvolume created and started')
+            mock_start_volume = MagicMock(return_value=True)
+            with patch.object(glusterfs, 'start_volume', mock_start_volume):
+                # Create, do not start
+                self.assertTrue(glusterfs.create_volume('newvolume',
+                                                        'host1:/brick'))
+                self.assertFalse(mock_start_volume.called)
 
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertEqual(glusterfs.create('newvolume', 'host1:/brick'),
-                             'Volume newvolume created. Start volume to use')
+                # Create and start
+                self.assertTrue(glusterfs.create_volume('newvolume',
+                                                        'host1:/brick',
+                                                        start=True))
+                self.assertTrue(mock_start_volume.called)
+
+                mock_start_volume.return_value = False
+                # Create and fail start
+                self.assertFalse(glusterfs.create_volume('newvolume',
+                                                         'host1:/brick',
+                                                         start=True))
+
+            mock_run.return_value = xml_command_fail
+            self.assertFalse(glusterfs.create_volume('newvolume', 'host1:/brick',
+                             True, True, True, 'tcp', True))
 
     # 'list_volumes' function tests: 1
 
@@ -558,10 +475,9 @@ class GlusterfsTestCase(TestCase):
         '''
         Test if it check the status of a gluster volume.
         '''
-        mock = MagicMock(return_value=xml_command_fail)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(
-                CommandExecutionError, glusterfs.status, 'myvol1')
+        mock_run = MagicMock(return_value=xml_command_fail)
+        with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+            self.assertIsNone(glusterfs.status('myvol1'))
 
         res = {'bricks': {
             'node01:/tmp/foo': {
@@ -630,26 +546,35 @@ class GlusterfsTestCase(TestCase):
         '''
         Test if it start a gluster volume.
         '''
-        mock_list = MagicMock(return_value=['Newvolume1', 'Newvolume2'])
-        with patch.object(glusterfs, 'list_volumes', mock_list):
-            mock_status = MagicMock(return_value={'status': '1'})
-            with patch.object(glusterfs, 'info', mock_status):
-                mock = MagicMock(return_value=xml_command_success)
-                with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                    self.assertEqual(glusterfs.start_volume('Newvolume1'),
-                                     'Volume already started')
+        # Stopped volume
+        mock_info = MagicMock(return_value={'Newvolume1': {'status': '0'}})
+        with patch.object(glusterfs, 'info', mock_info):
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertEqual(glusterfs.start_volume('Newvolume1'), True)
+                self.assertEqual(glusterfs.start_volume('nonExisting'), False)
+            mock_run = MagicMock(return_value=xml_command_fail)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertEqual(glusterfs.start_volume('Newvolume1'), False)
 
-            mock_status = MagicMock(return_value={'status': '0'})
-            with patch.object(glusterfs, 'info', mock_status):
-                mock_run = MagicMock(return_value=xml_command_success)
-                with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
-                    self.assertEqual(glusterfs.start_volume('Newvolume1'),
-                                     'Volume Newvolume1 started')
-
-        mock = MagicMock(return_value=xml_command_fail)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(
-                CommandExecutionError, glusterfs.start_volume, 'Newvolume1')
+        # Started volume
+        mock_info = MagicMock(return_value={'Newvolume1': {'status': '1'}})
+        with patch.object(glusterfs, 'info', mock_info):
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertEqual(
+                    glusterfs.start_volume('Newvolume1', force=True),
+                    True
+                )
+            mock_run = MagicMock(return_value=xml_command_fail)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                # cmd.run should not be called for already running volume:
+                self.assertEqual(glusterfs.start_volume('Newvolume1'), True)
+                # except when forcing:
+                self.assertEqual(
+                    glusterfs.start_volume('Newvolume1', force=True),
+                    False
+                )
 
     # 'stop_volume' function tests: 1
 
@@ -657,58 +582,63 @@ class GlusterfsTestCase(TestCase):
         '''
         Test if it stop a gluster volume.
         '''
-        mock = MagicMock(return_value={})
-        with patch.object(glusterfs, 'status', mock):
-            mock = MagicMock(return_value=xml_command_success)
-            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                self.assertEqual(glusterfs.stop_volume('Newvolume1'),
-                                 'Volume Newvolume1 stopped')
+        # Stopped volume
+        mock_info = MagicMock(return_value={'Newvolume1': {'status': '0'}})
+        with patch.object(glusterfs, 'info', mock_info):
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertEqual(glusterfs.stop_volume('Newvolume1'), True)
+                self.assertEqual(glusterfs.stop_volume('nonExisting'), False)
+            mock_run = MagicMock(return_value=xml_command_fail)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                # cmd.run should not be called for already stopped volume:
+                self.assertEqual(glusterfs.stop_volume('Newvolume1'), True)
 
-            mock = MagicMock(return_value=xml_command_fail)
-            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                self.assertRaises(
-                    CommandExecutionError, glusterfs.stop_volume, 'Newvolume1')
+        # Started volume
+        mock_info = MagicMock(return_value={'Newvolume1': {'status': '1'}})
+        with patch.object(glusterfs, 'info', mock_info):
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertEqual(glusterfs.stop_volume('Newvolume1'), True)
+                self.assertEqual(glusterfs.stop_volume('nonExisting'), False)
+            mock_run = MagicMock(return_value=xml_command_fail)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertEqual(glusterfs.stop_volume('Newvolume1'), False)
 
-        mock = MagicMock(return_value=xml_command_fail)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(
-                CommandExecutionError, glusterfs.stop_volume, 'Newvolume1')
+    # 'delete_volume' function tests: 1
 
-    # 'delete' function tests: 1
-
-    def test_delete(self):
+    def test_delete_volume(self):
         '''
         Test if it deletes a gluster volume.
         '''
-        mock = MagicMock(return_value=['Newvolume1', 'Newvolume2'])
-        with patch.object(glusterfs, 'list_volumes', mock):
+        mock_info = MagicMock(return_value={'Newvolume1': {'status': '1'}})
+        with patch.object(glusterfs, 'info', mock_info):
             # volume doesn't exist
-            self.assertRaises(
-                SaltInvocationError, glusterfs.delete, 'Newvolume3')
+            self.assertFalse(glusterfs.delete_volume('Newvolume3'))
 
-            mock = MagicMock(return_value={'status': '1'})
-            with patch.object(glusterfs, 'info', mock):
-                mock = MagicMock(return_value=xml_command_success)
-                # volume exists, should not be stopped, and is started
-                with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                    self.assertRaises(
-                        SaltInvocationError,
-                        glusterfs.delete,
-                        'Newvolume1',
-                        False)
+            mock_stop_volume = MagicMock(return_value=True)
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                with patch.object(glusterfs, 'stop_volume', mock_stop_volume):
+                    # volume exists, should not be stopped, and is started
+                    self.assertFalse(glusterfs.delete_volume('Newvolume1',
+                                                             False))
+                    self.assertFalse(mock_run.called)
+                    self.assertFalse(mock_stop_volume.called)
 
-                # volume exists, should be stopped, and is started
-                with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                    self.assertEqual(glusterfs.delete('Newvolume1'),
-                                     'Volume Newvolume1 stopped and deleted')
+                    # volume exists, should be stopped, and is started
+                    self.assertTrue(glusterfs.delete_volume('Newvolume1'))
+                    self.assertTrue(mock_run.called)
+                    self.assertTrue(mock_stop_volume.called)
 
-            # volume exists and isn't started
-            mock = MagicMock(return_value={'status': '0'})
-            with patch.object(glusterfs, 'info', mock):
-                mock = MagicMock(return_value=xml_command_success)
-                with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                    self.assertEqual(glusterfs.delete('Newvolume1'),
-                                     'Volume Newvolume1 deleted')
+        # volume exists and isn't started
+        mock_info = MagicMock(return_value={'Newvolume1': {'status': '2'}})
+        with patch.object(glusterfs, 'info', mock_info):
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                self.assertTrue(glusterfs.delete_volume('Newvolume1'))
+                mock_run.return_value = xml_command_fail
+                self.assertFalse(glusterfs.delete_volume('Newvolume1'))
 
     # 'add_volume_bricks' function tests: 1
 
@@ -716,45 +646,37 @@ class GlusterfsTestCase(TestCase):
         '''
         Test if it add brick(s) to an existing volume
         '''
-        # volume does not exist
-        mock = MagicMock(return_value=xml_command_fail)
-        with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-            self.assertRaises(
-                CommandExecutionError,
-                glusterfs.add_volume_bricks,
-                'Newvolume1',
-                ['bricks'])
+        mock_info = MagicMock(return_value={
+            'Newvolume1': {
+                'status': '1',
+                'bricks': {
+                    'brick1': {'path': 'host:/path1'},
+                    'brick2': {'path': 'host:/path2'}
+                }
+            }
+        })
+        with patch.object(glusterfs, 'info', mock_info):
+            mock_run = MagicMock(return_value=xml_command_success)
+            with patch.dict(glusterfs.__salt__, {'cmd.run': mock_run}):
+                # Volume does not exist
+                self.assertFalse(glusterfs.add_volume_bricks('nonExisting',
+                                                             ['bricks']))
+                # Brick already exists
+                self.assertTrue(glusterfs.add_volume_bricks('Newvolume1',
+                                                       ['host:/path2']))
+                # Already existing brick as a string
+                self.assertTrue(glusterfs.add_volume_bricks('Newvolume1',
+                                                            'host:/path2'))
+                self.assertFalse(mock_run.called)
+                # A new brick:
+                self.assertTrue(glusterfs.add_volume_bricks('Newvolume1',
+                                                            ['host:/new1']))
+                self.assertTrue(mock_run.called)
 
-        ret = '1 bricks successfully added to the volume Newvolume1'
-        # volume does exist
-        mock = MagicMock(return_value={'Newvolume1': {'bricks': {}}})
-        with patch.object(glusterfs, 'info', mock):
-            mock = MagicMock(return_value=xml_command_success)
-            # ... and the added brick does not exist
-            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                self.assertEqual(glusterfs.add_volume_bricks('Newvolume1',
-                                                             ['bricks']), ret)
-
-        mock = MagicMock(
-                return_value={'Newvolume1': {'bricks': {'brick1': {'path': 'bricks'}}}})
-        with patch.object(glusterfs, 'info', mock):
-            # ... and the added brick does exist
-            with patch.dict(glusterfs.__salt__, {'cmd.run': mock}):
-                # As a list
-                self.assertEqual(
-                    glusterfs.add_volume_bricks(
-                        'Newvolume1',
-                        ['bricks']),
-                    'Bricks already in volume Newvolume1')
-                # As a string
-                self.assertEqual(
-                    glusterfs.add_volume_bricks(
-                        'Newvolume1',
-                        'bricks'),
-                    'Bricks already in volume Newvolume1')
-                # And empty list
-                self.assertEqual(glusterfs.add_volume_bricks('Newvolume1', []),
-                                 'Bricks already in volume Newvolume1')
+                # Gluster call fails
+                mock_run.return_value = xml_command_fail
+                self.assertFalse(glusterfs.add_volume_bricks('Newvolume1',
+                                                             ['new:/path']))
 
 
 if __name__ == '__main__':
