@@ -32,7 +32,7 @@ grainsmod.__opts__ = {
 grainsmod.__salt__ = {}
 
 
-@patch.dict(grainsmod.__salt__, {'saltutil.sync_grains': MagicMock()})
+@patch.dict(grainsmod.__salt__, {'saltutil.refresh_grains': MagicMock()})
 @skipIf(NO_MOCK, NO_MOCK_REASON)
 class GrainsModuleTestCase(TestCase):
 
