@@ -202,6 +202,26 @@ class SnapperTestCase(TestCase):
         self.assertEqual(snapper.status_to_string(128), ["extended attributes changed"])
         self.assertEqual(snapper.status_to_string(256), ["ACL info changed"])
 
+    @patch('salt.modules.snapper.snapper.CreateConfig', MagicMock())
+    @patch('salt.modules.snapper.snapper.GetConfig', MagicMock(return_value=DBUS_RET['ListConfigs'][0]))
+    def test_create_config(self):
+        opts = {
+            'name': 'testconfig',
+            'subvolume': '/foo/bar/',
+            'fstype': 'btrfs',
+            'template': 'mytemplate',
+            'extra_opts': {"NUMBER_CLEANUP": False},
+        }
+        with patch('salt.modules.snapper.set_config', MagicMock()) as set_config_mock:
+            self.assertEqual(snapper.create_config(**opts), DBUS_RET['ListConfigs'][0])
+            set_config_mock.assert_called_with("testconfig", **opts['extra_opts'])
+
+        with patch('salt.modules.snapper.set_config', MagicMock()) as set_config_mock:
+            del opts['extra_opts']
+            self.assertEqual(snapper.create_config(**opts), DBUS_RET['ListConfigs'][0])
+            assert not set_config_mock.called
+            self.assertRaises(CommandExecutionError, snapper.create_config)
+
     @patch('salt.modules.snapper.snapper.CreateSingleSnapshot', MagicMock(return_value=1234))
     @patch('salt.modules.snapper.snapper.CreatePreSnapshot', MagicMock(return_value=1234))
     @patch('salt.modules.snapper.snapper.CreatePostSnapshot', MagicMock(return_value=1234))
@@ -216,6 +236,36 @@ class SnapperTestCase(TestCase):
             }
             self.assertEqual(snapper.create_snapshot(**opts), 1234)
 
+    @patch('salt.modules.snapper.snapper.DeleteSnapshots', MagicMock())
+    @patch('salt.modules.snapper.snapper.ListSnapshots', MagicMock(return_value=DBUS_RET['ListSnapshots']))
+    def test_delete_snapshot_id_success(self):
+        self.assertEqual(snapper.delete_snapshot(snapshots_ids=43), {"root": {"ids": [43], "status": "deleted"}})
+        self.assertEqual(snapper.delete_snapshot(snapshots_ids=[42, 43]), {"root": {"ids": [42, 43], "status": "deleted"}})
+
+    @patch('salt.modules.snapper.snapper.DeleteSnapshots', MagicMock())
+    @patch('salt.modules.snapper.snapper.ListSnapshots', MagicMock(return_value=DBUS_RET['ListSnapshots']))
+    def test_delete_snapshot_id_fail(self):
+        self.assertRaises(CommandExecutionError, snapper.delete_snapshot)
+        self.assertRaises(CommandExecutionError, snapper.delete_snapshot, snapshots_ids=1)
+        self.assertRaises(CommandExecutionError, snapper.delete_snapshot, snapshots_ids=[1, 2])
+
+    @patch('salt.modules.snapper.snapper.SetSnapshot', MagicMock())
+    def test_modify_snapshot(self):
+        _ret = {
+            'userdata': {'userdata2': 'uservalue2'},
+            'description': 'UPDATED DESCRIPTION', 'timestamp': 1457006571,
+            'cleanup': 'number', 'user': 'root', 'type': 'pre', 'id': 42
+        }
+        _opts = {
+            'config': 'root',
+            'snapshot_id': 42,
+            'cleanup': 'number',
+            'description': 'UPDATED DESCRIPTION',
+            'userdata': {'userdata2': 'uservalue2'},
+        }
+        with patch('salt.modules.snapper.get_snapshot', MagicMock(side_effect=[DBUS_RET['ListSnapshots'][0], _ret])):
+            self.assertDictEqual(snapper.modify_snapshot(**_opts), _ret)
+
     @patch('salt.modules.snapper._get_last_snapshot', MagicMock(return_value={'id': 42}))
     def test__get_num_interval(self):
         self.assertEqual(snapper._get_num_interval(config=None, num_pre=None, num_post=None), (42, 0))  # pylint: disable=protected-access
